import argparse
from urllib.request import urlopen
from decimal import Decimal
import sys
import logging
from PyQt5.QtWidgets import QApplication
from xulpymoney.libxulpymoney import MemSources
from xulpymoney.libxulpymoneyfunctions import addDebugSystem, addCommonToArgParse

class CurrentPriceTickerFund:
    def __init__(self,ticker, xulpymoney, stockmarket):
        self.ticker=ticker
        self.stockmarket=stockmarket
        self.xulpymoney=xulpymoney
        self.dtaware=self.stockmarket.estimated_datetime_for_daily_quote()
        self.price=None

    def __repr__(self):
        if self.xulpymoney!=None:
            return "PRICE | XULPYMONEY | {} | {} | {}".format(self.xulpymoney, self.dtaware, self.price)
        else:
            return "PRICE | STOCKMARKET | XX | TICKER | {} | {} | {}".format(self.ticker, self.dtaware , self.price)

    def get_price(self):
        try:
            url='http://www.morningstar.es/es/funds/snapshot/snapshot.aspx?id='+self.ticker
            logging.debug(url)
            web=urlopen(url)
        except:
            print ("ERROR | ERROR LOADING MORNINGSTAR PAGE")
            sys.exit(255)
            
        if web==None:
            print ("ERROR | FETCHED EMPTY PAGE")
            sys.exit(255)

        for l in web.readlines():
            l=l.decode('UTF-8')
            if l.find("Estadística Rápida")!=-1:
                self.price=Decimal(l.split('line text">')[1].split("</td")[0].split("\xa0")[1].replace(",","."))
                return
        print ("ERROR | ERROR PARSING")
        
## Class to parse etf in morningstar
class CurrentPriceTickerETF(CurrentPriceTickerFund):
    def __init__(self, ticker, xulpymoney, stockmarket):
        CurrentPriceTickerFund.__init__(self, ticker, xulpymoney, stockmarket)
        
    def stockmarket2url(self):
        if self.stockmarket.id==3:#Paris
            return "xpar"
            
    ## #Returns LVE de LVE.PA
    def yahoo2url(self):
        return self.ticker.split(".")[0].lower()
        
    def get_price(self):
        try:
            url='https://www.morningstar.com/etfs/{}/{}/quote.html'.format(self.stockmarket2url(), self.yahoo2url())
            logging.debug(url)
            web=urlopen(url)
        except:
            print ("ERROR | ERROR LOADING MORNINGSTAR PAGE")
            sys.exit(255)
            
        if web==None:
            print ("ERROR | FETCHED EMPTY PAGE")
            sys.exit(255)

        for l in web.readlines():
            l=l.decode('UTF-8')
            if l.find("25")!=-1:
                print (l)
#                self.price=Decimal(l.split('line text">')[1].split("</td")[0].split("\xa0")[1].replace(",","."))
#                return
        print ("ERROR | ERROR PARSING")
def main():
    app = QApplication(sys.argv)
    parser=argparse.ArgumentParser()
    parser.add_argument('--TICKER_XULPYMONEY', help='XULPYMONEY code', nargs=2, metavar="VALUE")
    parser.add_argument('--STOCKMARKET', help='Stock market id', metavar="ID", required=True)   
    group2=parser.add_mutually_exclusive_group(required=True)
    group2.add_argument('--fund', help="Fund search", action='store_true', default=False)
    group2.add_argument('--etf', help="ETF search", action='store_true', default=False) 
    addCommonToArgParse(parser)
    args=parser.parse_args() 
    addDebugSystem(args)

    mem=MemSources()
    stockmarket=mem.stockmarkets.find_by_id(int(args.STOCKMARKET))
    logging.info(stockmarket)

    if args.TICKER_XULPYMONEY and args.fund==True:
        s=CurrentPriceTickerFund(args.TICKER_XULPYMONEY[0], args.TICKER_XULPYMONEY[1], stockmarket)
        s.get_price()
        print(s)
    elif args.TICKER_XULPYMONEY and args.etf==True:
        s=CurrentPriceTickerETF(args.TICKER_XULPYMONEY[0], args.TICKER_XULPYMONEY[1], stockmarket)
        s.get_price()
        print(s)




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/frmDividendsAdd.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_frmDividendsAdd(object):
    def setupUi(self, frmDividendsAdd):
        frmDividendsAdd.setObjectName("frmDividendsAdd")
        frmDividendsAdd.setWindowModality(QtCore.Qt.WindowModal)
        frmDividendsAdd.resize(488, 493)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/xulpymoney/dividends.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        frmDividendsAdd.setWindowIcon(icon)
        frmDividendsAdd.setModal(True)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(frmDividendsAdd)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.lblTitulo = QtWidgets.QLabel(frmDividendsAdd)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.lblTitulo.setFont(font)
        self.lblTitulo.setStyleSheet("color: rgb(0, 128, 0);")
        self.lblTitulo.setAlignment(QtCore.Qt.AlignCenter)
        self.lblTitulo.setObjectName("lblTitulo")
        self.verticalLayout.addWidget(self.lblTitulo)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.wdgDT = wdgDatetime(frmDividendsAdd)
        self.wdgDT.setObjectName("wdgDT")
        self.horizontalLayout.addWidget(self.wdgDT)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.label = QtWidgets.QLabel(frmDividendsAdd)
        self.label.setObjectName("label")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label)
        self.cmb = QtWidgets.QComboBox(frmDividendsAdd)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cmb.sizePolicy().hasHeightForWidth())
        self.cmb.setSizePolicy(sizePolicy)
        self.cmb.setObjectName("cmb")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.cmb)
        self.lblGross = QtWidgets.QLabel(frmDividendsAdd)
        self.lblGross.setObjectName("lblGross")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.lblGross)
        self.txtBruto = myQLineEdit(frmDividendsAdd)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtBruto.sizePolicy().hasHeightForWidth())
        self.txtBruto.setSizePolicy(sizePolicy)
        self.txtBruto.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.txtBruto.setObjectName("txtBruto")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.txtBruto)
        self.lblGrossAccount = QtWidgets.QLabel(frmDividendsAdd)
        self.lblGrossAccount.setObjectName("lblGrossAccount")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.lblGrossAccount)
        self.wdgCurrencyConversion = wdgCurrencyConversion(frmDividendsAdd)
        self.wdgCurrencyConversion.setObjectName("wdgCurrencyConversion")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.wdgCurrencyConversion)
        self.lblLiquido_4 = QtWidgets.QLabel(frmDividendsAdd)
        self.lblLiquido_4.setObjectName("lblLiquido_4")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.lblLiquido_4)
        self.txtRetencion = myQLineEdit(frmDividendsAdd)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtRetencion.sizePolicy().hasHeightForWidth())
        self.txtRetencion.setSizePolicy(sizePolicy)
        self.txtRetencion.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.txtRetencion.setObjectName("txtRetencion")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.txtRetencion)
        self.lblLiquido_5 = QtWidgets.QLabel(frmDividendsAdd)
        self.lblLiquido_5.setObjectName("lblLiquido_5")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.lblLiquido_5)
        self.txtDPA = myQLineEdit(frmDividendsAdd)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtDPA.sizePolicy().hasHeightForWidth())
        self.txtDPA.setSizePolicy(sizePolicy)
        self.txtDPA.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.txtDPA.setObjectName("txtDPA")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.txtDPA)
        self.lblLiquido = QtWidgets.QLabel(frmDividendsAdd)
        self.lblLiquido.setObjectName("lblLiquido")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.LabelRole, self.lblLiquido)
        self.txtComision = myQLineEdit(frmDividendsAdd)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtComision.sizePolicy().hasHeightForWidth())
        self.txtComision.setSizePolicy(sizePolicy)
        self.txtComision.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.txtComision.setObjectName("txtComision")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.FieldRole, self.txtComision)
        self.lblLiquido_7 = QtWidgets.QLabel(frmDividendsAdd)
        self.lblLiquido_7.setObjectName("lblLiquido_7")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.LabelRole, self.lblLiquido_7)
        self.txtNeto = myQLineEdit(frmDividendsAdd)
        self.txtNeto.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtNeto.sizePolicy().hasHeightForWidth())
        self.txtNeto.setSizePolicy(sizePolicy)
        self.txtNeto.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.txtNeto.setReadOnly(True)
        self.txtNeto.setObjectName("txtNeto")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.FieldRole, self.txtNeto)
        self.verticalLayout.addLayout(self.formLayout)
        self.lblTPC = QtWidgets.QLabel(frmDividendsAdd)
        self.lblTPC.setText("")
        self.lblTPC.setAlignment(QtCore.Qt.AlignCenter)
        self.lblTPC.setObjectName("lblTPC")
        self.verticalLayout.addWidget(self.lblTPC)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem2)
        self.cmd = QtWidgets.QPushButton(frmDividendsAdd)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/xulpymoney/save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmd.setIcon(icon1)
        self.cmd.setObjectName("cmd")
        self.horizontalLayout_2.addWidget(self.cmd)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem3)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3.addLayout(self.verticalLayout)

        self.retranslateUi(frmDividendsAdd)
        QtCore.QMetaObject.connectSlotsByName(frmDividendsAdd)
        frmDividendsAdd.setTabOrder(self.cmb, self.txtBruto)
        frmDividendsAdd.setTabOrder(self.txtBruto, self.txtRetencion)
        frmDividendsAdd.setTabOrder(self.txtRetencion, self.txtDPA)
        frmDividendsAdd.setTabOrder(self.txtDPA, self.txtComision)
        frmDividendsAdd.setTabOrder(self.txtComision, self.cmd)
        frmDividendsAdd.setTabOrder(self.cmd, self.txtNeto)

    def retranslateUi(self, frmDividendsAdd):
        _translate = QtCore.QCoreApplication.translate
        frmDividendsAdd.setWindowTitle(_translate("frmDividendsAdd", "Dividends"))
        self.lblTitulo.setText(_translate("frmDividendsAdd", "New dividend"))
        self.label.setText(_translate("frmDividendsAdd", "Select dividend type"))
        self.lblGross.setText(_translate("frmDividendsAdd", "Gross amount"))
        self.txtBruto.setText(_translate("frmDividendsAdd", "0"))
        self.lblGrossAccount.setText(_translate("frmDividendsAdd", "Gross amount in account currency"))
        self.lblLiquido_4.setText(_translate("frmDividendsAdd", "Withholding tax amount"))
        self.txtRetencion.setText(_translate("frmDividendsAdd", "0"))
        self.lblLiquido_5.setText(_translate("frmDividendsAdd", "Dividend per share"))
        self.txtDPA.setText(_translate("frmDividendsAdd", "0"))
        self.lblLiquido.setText(_translate("frmDividendsAdd", "Comission"))
        self.txtComision.setText(_translate("frmDividendsAdd", "0"))
        self.lblLiquido_7.setText(_translate("frmDividendsAdd", "Net amount"))
        self.txtNeto.setText(_translate("frmDividendsAdd", "0"))


from xulpymoney.ui.myqlineedit import myQLineEdit
from xulpymoney.ui.wdgCurrencyConversion import wdgCurrencyConversion
from xulpymoney.ui.wdgDatetime import wdgDatetime
import xulpymoney.images.xulpymoney_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/frmCreditCardsAdd.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_frmCreditCardsAdd(object):
    def setupUi(self, frmCreditCardsAdd):
        frmCreditCardsAdd.setObjectName("frmCreditCardsAdd")
        frmCreditCardsAdd.setWindowModality(QtCore.Qt.WindowModal)
        frmCreditCardsAdd.resize(426, 225)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/xulpymoney/visa2.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        frmCreditCardsAdd.setWindowIcon(icon)
        frmCreditCardsAdd.setModal(True)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout(frmCreditCardsAdd)
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.lblTitle = QtWidgets.QLabel(frmCreditCardsAdd)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.lblTitle.setFont(font)
        self.lblTitle.setStyleSheet("color: rgb(0, 128, 0);")
        self.lblTitle.setAlignment(QtCore.Qt.AlignCenter)
        self.lblTitle.setObjectName("lblTitle")
        self.verticalLayout.addWidget(self.lblTitle)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_5 = QtWidgets.QLabel(frmCreditCardsAdd)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_5.addWidget(self.label_5)
        self.txtCreditCard = QtWidgets.QLineEdit(frmCreditCardsAdd)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtCreditCard.sizePolicy().hasHeightForWidth())
        self.txtCreditCard.setSizePolicy(sizePolicy)
        self.txtCreditCard.setObjectName("txtCreditCard")
        self.horizontalLayout_5.addWidget(self.txtCreditCard)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.chkDelayed = QtWidgets.QCheckBox(frmCreditCardsAdd)
        self.chkDelayed.setObjectName("chkDelayed")
        self.horizontalLayout.addWidget(self.chkDelayed)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.label_7 = QtWidgets.QLabel(frmCreditCardsAdd)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_7.addWidget(self.label_7)
        self.txtMaximum = myQLineEdit(frmCreditCardsAdd)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtMaximum.sizePolicy().hasHeightForWidth())
        self.txtMaximum.setSizePolicy(sizePolicy)
        self.txtMaximum.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.txtMaximum.setObjectName("txtMaximum")
        self.horizontalLayout_7.addWidget(self.txtMaximum)
        self.verticalLayout.addLayout(self.horizontalLayout_7)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_4 = QtWidgets.QLabel(frmCreditCardsAdd)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_4.addWidget(self.label_4)
        self.txtNumber = QtWidgets.QLineEdit(frmCreditCardsAdd)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtNumber.sizePolicy().hasHeightForWidth())
        self.txtNumber.setSizePolicy(sizePolicy)
        self.txtNumber.setObjectName("txtNumber")
        self.horizontalLayout_4.addWidget(self.txtNumber)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.cmd = QtWidgets.QPushButton(frmCreditCardsAdd)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/xulpymoney/save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmd.setIcon(icon1)
        self.cmd.setObjectName("cmd")
        self.verticalLayout.addWidget(self.cmd)
        self.horizontalLayout_8.addLayout(self.verticalLayout)

        self.retranslateUi(frmCreditCardsAdd)
        QtCore.QMetaObject.connectSlotsByName(frmCreditCardsAdd)

    def retranslateUi(self, frmCreditCardsAdd):
        _translate = QtCore.QCoreApplication.translate
        frmCreditCardsAdd.setWindowTitle(_translate("frmCreditCardsAdd", "New credit card"))
        self.label_5.setText(_translate("frmCreditCardsAdd", "Credit card name"))
        self.chkDelayed.setText(_translate("frmCreditCardsAdd", "Has it delayed payment?"))
        self.label_7.setText(_translate("frmCreditCardsAdd", "Maximum balance"))
        self.txtMaximum.setText(_translate("frmCreditCardsAdd", "0"))
        self.label_4.setText(_translate("frmCreditCardsAdd", "Credit card number"))
        self.cmd.setText(_translate("frmCreditCardsAdd", "Save"))


from xulpymoney.ui.myqlineedit import myQLineEdit
import xulpymoney.images.xulpymoney_rc



from PyQt5.QtWidgets import QDialog
from decimal import Decimal
from xulpymoney.hlcontracts import HlContract
from xulpymoney.libxulpymoneyfunctions import qmessagebox
from xulpymoney.ui.Ui_frmHlContractAdd import Ui_frmHlContractAdd

class frmHlContractAdd(QDialog, Ui_frmHlContractAdd):
    def __init__(self, mem, investment, hlcontract=None,  parent=None):
        """
        Si dividend es None se insertar
        Si dividend es un objeto se modifica"""
        QDialog.__init__(self, parent)
        self.setupUi(self)
        self.mem=mem
        self.investment=investment
        self.hlcontract=hlcontract

        self.wdgDT.show_microseconds(False)
        self.wdgDT.show_timezone(False)
        if self.hlcontract==None:#insert
            self.hlcontract=HlContract(self.mem, self.investment)
            self.cmd.setText(self.tr("Add new High-Low contract"))
            self.wdgDT.set(self.mem, None, self.mem.localzone)
        else:#update
            self.wdgDT.set(self.mem, self.hlcontract.datetime, self.mem.localzone)
            self.txtGuarantee.setText(self.hlcontract.guarantee)
            self.txtAdjustment.setText(self.hlcontract.adjustment)
            self.txtInterest.setText(self.hlcontract.interest)
            self.txtCommission.setText(self.hlcontract.commission)
            self.cmd.setText(self.tr("Update High-Low contract"))

    def on_cmd_pressed(self):                        
        if self.txtCommission.decimal()<Decimal('0'):
            qmessagebox(self.tr("Commissions can't be a negative amount"))
            return
        
        try:
            self.hlcontract.guarantee=self.txtGuarantee.decimal()
            self.hlcontract.adjustment=self.txtAdjustment.decimal()
            self.hlcontract.commission=self.txtCommission.decimal()
            self.hlcontract.interest=self.txtInterest.decimal()
            self.hlcontract.datetime=self.wdgDT.datetime()
        except:
            qmessagebox(self.tr("Data error. Please check them."))
            return

        self.hlcontract.save()
        self.mem.con.commit()
        self.done(0)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/frmAccountOperationsAdd.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_frmAccountOperationsAdd(object):
    def setupUi(self, frmAccountOperationsAdd):
        frmAccountOperationsAdd.setObjectName("frmAccountOperationsAdd")
        frmAccountOperationsAdd.setWindowModality(QtCore.Qt.WindowModal)
        frmAccountOperationsAdd.resize(566, 286)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/xulpymoney/document-edit.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        frmAccountOperationsAdd.setWindowIcon(icon)
        frmAccountOperationsAdd.setModal(True)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(frmAccountOperationsAdd)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.lblTitulo = QtWidgets.QLabel(frmAccountOperationsAdd)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.lblTitulo.setFont(font)
        self.lblTitulo.setStyleSheet("color: rgb(0, 128, 0);")
        self.lblTitulo.setAlignment(QtCore.Qt.AlignCenter)
        self.lblTitulo.setObjectName("lblTitulo")
        self.verticalLayout.addWidget(self.lblTitulo)
        self.wdgDT = wdgDatetime(frmAccountOperationsAdd)
        self.wdgDT.setObjectName("wdgDT")
        self.verticalLayout.addWidget(self.wdgDT)
        self.groupBox = QtWidgets.QGroupBox(frmAccountOperationsAdd)
        self.groupBox.setTitle("")
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.radAccounts = QtWidgets.QRadioButton(self.groupBox)
        self.radAccounts.setObjectName("radAccounts")
        self.horizontalLayout_5.addWidget(self.radAccounts)
        self.cmbAccounts = QtWidgets.QComboBox(self.groupBox)
        self.cmbAccounts.setObjectName("cmbAccounts")
        self.horizontalLayout_5.addWidget(self.cmbAccounts)
        self.verticalLayout_3.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.radCreditCards = QtWidgets.QRadioButton(self.groupBox)
        self.radCreditCards.setObjectName("radCreditCards")
        self.horizontalLayout_6.addWidget(self.radCreditCards)
        self.cmbCreditCards = QtWidgets.QComboBox(self.groupBox)
        self.cmbCreditCards.setObjectName("cmbCreditCards")
        self.horizontalLayout_6.addWidget(self.cmbCreditCards)
        self.verticalLayout_3.addLayout(self.horizontalLayout_6)
        self.verticalLayout.addWidget(self.groupBox)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_2 = QtWidgets.QLabel(frmAccountOperationsAdd)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_3.addWidget(self.label_2)
        self.cmbConcepts = QtWidgets.QComboBox(frmAccountOperationsAdd)
        self.cmbConcepts.setObjectName("cmbConcepts")
        self.horizontalLayout_3.addWidget(self.cmbConcepts)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_3 = QtWidgets.QLabel(frmAccountOperationsAdd)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout.addWidget(self.label_3)
        self.txtImporte = myQLineEdit(frmAccountOperationsAdd)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtImporte.sizePolicy().hasHeightForWidth())
        self.txtImporte.setSizePolicy(sizePolicy)
        self.txtImporte.setObjectName("txtImporte")
        self.horizontalLayout.addWidget(self.txtImporte)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_4 = QtWidgets.QLabel(frmAccountOperationsAdd)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_4.addWidget(self.label_4)
        self.txtComentario = QtWidgets.QLineEdit(frmAccountOperationsAdd)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtComentario.sizePolicy().hasHeightForWidth())
        self.txtComentario.setSizePolicy(sizePolicy)
        self.txtComentario.setObjectName("txtComentario")
        self.horizontalLayout_4.addWidget(self.txtComentario)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.cmd = QtWidgets.QPushButton(frmAccountOperationsAdd)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/xulpymoney/save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmd.setIcon(icon1)
        self.cmd.setObjectName("cmd")
        self.verticalLayout.addWidget(self.cmd)
        self.verticalLayout_2.addLayout(self.verticalLayout)

        self.retranslateUi(frmAccountOperationsAdd)
        self.radAccounts.toggled['bool'].connect(self.cmbCreditCards.setDisabled)
        self.radCreditCards.toggled['bool'].connect(self.cmbAccounts.setDisabled)
        QtCore.QMetaObject.connectSlotsByName(frmAccountOperationsAdd)

    def retranslateUi(self, frmAccountOperationsAdd):
        _translate = QtCore.QCoreApplication.translate
        frmAccountOperationsAdd.setWindowTitle(_translate("frmAccountOperationsAdd", "New account operation"))
        self.radAccounts.setText(_translate("frmAccountOperationsAdd", "Se&lect an account"))
        self.radCreditCards.setText(_translate("frmAccountOperationsAdd", "Selec&t a credit card"))
        self.label_2.setText(_translate("frmAccountOperationsAdd", "Select a concept"))
        self.label_3.setText(_translate("frmAccountOperationsAdd", "Add an amount"))
        self.label_4.setText(_translate("frmAccountOperationsAdd", "Add a comment"))
        self.cmd.setText(_translate("frmAccountOperationsAdd", "Save"))


from xulpymoney.ui.myqlineedit import myQLineEdit
from xulpymoney.ui.wdgDatetime import wdgDatetime
import xulpymoney.images.xulpymoney_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/wdgAccounts.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_wdgAccounts(object):
    def setupUi(self, wdgAccounts):
        wdgAccounts.setObjectName("wdgAccounts")
        wdgAccounts.resize(579, 318)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(wdgAccounts)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.lbl = QtWidgets.QLabel(wdgAccounts)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.lbl.setFont(font)
        self.lbl.setAlignment(QtCore.Qt.AlignCenter)
        self.lbl.setObjectName("lbl")
        self.verticalLayout.addWidget(self.lbl)
        self.chkInactivas = QtWidgets.QCheckBox(wdgAccounts)
        self.chkInactivas.setObjectName("chkInactivas")
        self.verticalLayout.addWidget(self.chkInactivas)
        self.tblAccounts = myQTableWidget(wdgAccounts)
        self.tblAccounts.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblAccounts.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tblAccounts.setAlternatingRowColors(True)
        self.tblAccounts.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tblAccounts.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tblAccounts.setObjectName("tblAccounts")
        self.tblAccounts.setColumnCount(4)
        self.tblAccounts.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tblAccounts.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblAccounts.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblAccounts.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblAccounts.setHorizontalHeaderItem(3, item)
        self.tblAccounts.verticalHeader().setVisible(False)
        self.tblAccounts.verticalHeader().setCascadingSectionResizes(False)
        self.verticalLayout.addWidget(self.tblAccounts)
        self.verticalLayout_2.addLayout(self.verticalLayout)
        self.wdgIBM = QtWidgets.QWidget(wdgAccounts)
        self.wdgIBM.setObjectName("wdgIBM")
        self.verticalLayout_2.addWidget(self.wdgIBM)
        self.verticalLayout_3.addLayout(self.verticalLayout_2)
        self.lblTotal = QtWidgets.QLabel(wdgAccounts)
        font = QtGui.QFont()
        font.setPointSize(8)
        font.setBold(True)
        font.setWeight(75)
        self.lblTotal.setFont(font)
        self.lblTotal.setText("")
        self.lblTotal.setAlignment(QtCore.Qt.AlignCenter)
        self.lblTotal.setObjectName("lblTotal")
        self.verticalLayout_3.addWidget(self.lblTotal)
        self.actionAccountAdd = QtWidgets.QAction(wdgAccounts)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/xulpymoney/add.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionAccountAdd.setIcon(icon)
        self.actionAccountAdd.setObjectName("actionAccountAdd")
        self.actionAccountReport = QtWidgets.QAction(wdgAccounts)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/xulpymoney/coins.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionAccountReport.setIcon(icon1)
        self.actionAccountReport.setObjectName("actionAccountReport")
        self.actionAccountDelete = QtWidgets.QAction(wdgAccounts)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/xulpymoney/list-remove.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionAccountDelete.setIcon(icon2)
        self.actionAccountDelete.setObjectName("actionAccountDelete")
        self.actionActive = QtWidgets.QAction(wdgAccounts)
        self.actionActive.setCheckable(True)
        self.actionActive.setObjectName("actionActive")
        self.actionTransfer = QtWidgets.QAction(wdgAccounts)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/xulpymoney/transfer.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionTransfer.setIcon(icon3)
        self.actionTransfer.setObjectName("actionTransfer")

        self.retranslateUi(wdgAccounts)
        QtCore.QMetaObject.connectSlotsByName(wdgAccounts)

    def retranslateUi(self, wdgAccounts):
        _translate = QtCore.QCoreApplication.translate
        self.lbl.setText(_translate("wdgAccounts", "Account list"))
        self.chkInactivas.setText(_translate("wdgAccounts", "Show inactive accounts"))
        item = self.tblAccounts.horizontalHeaderItem(0)
        item.setText(_translate("wdgAccounts", "Account"))
        item = self.tblAccounts.horizontalHeaderItem(1)
        item.setText(_translate("wdgAccounts", "Bank"))
        item = self.tblAccounts.horizontalHeaderItem(2)
        item.setText(_translate("wdgAccounts", "Account number"))
        item = self.tblAccounts.horizontalHeaderItem(3)
        item.setText(_translate("wdgAccounts", "Balance"))
        self.actionAccountAdd.setText(_translate("wdgAccounts", "New account"))
        self.actionAccountAdd.setToolTip(_translate("wdgAccounts", "New account"))
        self.actionAccountReport.setText(_translate("wdgAccounts", "Account report"))
        self.actionAccountReport.setToolTip(_translate("wdgAccounts", "Account report"))
        self.actionAccountDelete.setText(_translate("wdgAccounts", "Delete account"))
        self.actionAccountDelete.setToolTip(_translate("wdgAccounts", "Delete account"))
        self.actionActive.setText(_translate("wdgAccounts", "Is it active?"))
        self.actionActive.setToolTip(_translate("wdgAccounts", "Is it active?"))
        self.actionTransfer.setText(_translate("wdgAccounts", "Transfer between accounts"))
        self.actionTransfer.setToolTip(_translate("wdgAccounts", "Transfer between accounts"))


from xulpymoney.ui.myqtablewidget import myQTableWidget
import xulpymoney.images.xulpymoney_rc



## @namespace xulpymoney.libcounter
## This file is from xulpymoney project
## Do not edit, It will be overriden

import datetime
import math
import platform
import sys
from colorama import Style, Fore, init

init(autoreset=True)

class Counter:
    def __init__(self, maxsteps):
        self.__current=0
        self.__maxsteps=maxsteps
        self.__start=datetime.datetime.now()
        self.__end=None
        self.setName("Counter")
        self.setSameLine(True)

    ## Gets start datetime. This parameter is set in Constructor
    ## @returns datetime
    def start(self):
        return self.__start
        
    ## Gets the end time of the counter. It's set when final message function is invoked
    ## return datetime or None.  If counter is working this value returns None
    def end(self):
        return self.__end
        
    ## Gets the number of max steps of the counter
    ## @return integer
    def maxSteps(self):
        return self.__maxsteps
    ## Sets if output must be shown in a line
    def setSameLine(self, bool):
        self.__sameline=bool

    def name(self):
        return self.__name

    def setName(self, name):
        self.__name=name
        
    ## Converts a number of seconds into a pretty string
    ## @param seconds Integer with the number of seconds
    ## @return string
    @staticmethod
    def seconds2pretty(segundos):
        dias=int(segundos/(24*60*60))
        segundosquedan=math.fmod(segundos,24*60*60)
        horas=int(segundosquedan/(60*60))
        segundosquedan=math.fmod(segundosquedan,60*60)
        minutos=int(segundosquedan/60)
        segundosquedan=math.fmod(segundosquedan,60)
        segundos=int(segundosquedan)
        return "{0}d {1}h {2}m {3}s".format(dias,  horas,  minutos, segundos)
        
    ## Function that returns the number of seconds remaining to end Counter
    def seconds_estimated_resting(self):
        """
            Función que devuelve segundos estimados que quedan
        """
        if self.__current==0:
            return 0
        resultado=(self.maxSteps()-self.__current)*(datetime.datetime.now()-self.start()).total_seconds()/self.__current
        return resultado    
        
    ## Function that returns the number of seconds the counter will last
    def seconds_estimated(self):
        if self.__current==0:
            return 0
        resultado=self.maxSteps()*(datetime.datetime.now()-self.start()).total_seconds()/self.__current
        return resultado
        
    ## Function that returns the number of seconds elapsed
    def seconds_currrent(self):
        """Tiempo actual"""
        return (datetime.datetime.now()-self.start()).total_seconds()
        
    def next_step(self):
        self.__current=self.__current+1
        if self.__current>self.maxSteps():
            print()
            print (Style.BRIGHT+Fore.RED+ "You need to change counter maximum steps in the Counter constructor to {}".format(self.__current))
        elif self.__current==self.maxSteps():
            self.__message_final()
        else:
            self.__message_step()
        
    def tpc_completado(self):
        if self.maxSteps()==0:
            return int(0)
        return int(100*self.__current/self.maxSteps())

    def __message_step(self):
        global parser
        if platform.system()=="Windows":
            tpc_completado=self.tpc_completado()
            segundos_current=Counter.seconds2pretty(self.seconds_currrent())
            segundos_estimados=Counter.seconds2pretty(self.seconds_estimated())
        else:
            tpc_completado=Style.BRIGHT+Fore.GREEN + str(self.tpc_completado())+ Style.NORMAL+ Fore.WHITE
            segundos_current=Style.BRIGHT+Fore.GREEN + Counter.seconds2pretty(self.seconds_currrent())+ Style.NORMAL+ Fore.WHITE
            segundos_estimados=Style.BRIGHT+Fore.RED + Counter.seconds2pretty(self.seconds_estimated())+ Style.NORMAL+ Fore.WHITE
        s="{}. Completado {} %. Tiempo transcurrido: {}. Tiempo estimado: {}. ".format(Fore.YELLOW + self.name()+Fore.RESET, tpc_completado, segundos_current, segundos_estimados)
        if self.__sameline==True:
            sys.stdout.write("\b"*(len(s)+10))
            sys.stdout.write(s)
            sys.stdout.flush()
        else:
            print(s)

    def __message_final(self):
        if self.__sameline==True:
            sys.stdout.flush()
            print("\n")
        self.__end=datetime.datetime.now()
        print("El proceso duró {}".format(Style.BRIGHT+Fore.RED+Counter.seconds2pretty(self.seconds_currrent())+ Style.NORMAL+ Fore.WHITE))


if __name__ == "__main__":
    import time
    print (Fore.GREEN + "This is a counter in the same line")
    example=Counter(10)
    example.setName("Counter in the same line")
    for x in range(example.maxSteps()):
        time.sleep(0.1)
        example.next_step()
        
    print()
    print (Fore.GREEN + "This is a normal counter")
    example=Counter(10)
    example.setName("Normal counter")
    example.setSameLine(False)
    for x in range(example.maxSteps()):
        time.sleep(0.1)
        example.next_step()
        
    print()
    print (Fore.GREEN + "This is a counter with bad max steps")
    example=Counter(9)
    example.setName("Counter in the same line")
    for x in range(10):
        time.sleep(0.1)
        example.next_step()
    print()



import datetime
from PyQt5.QtCore import pyqtSlot
from PyQt5.QtWidgets import QWidget, QDialog, QVBoxLayout, QMenu, QMessageBox
from xulpymoney.ui.Ui_wdgOpportunities import Ui_wdgOpportunities
from xulpymoney.opportunities import OpportunityManager
from xulpymoney.ui.wdgOpportunitiesAdd import wdgOpportunitiesAdd
from xulpymoney.ui.wdgProductHistoricalChart import wdgProductHistoricalOpportunity
from xulpymoney.ui.wdgCalculator import wdgCalculator
from decimal import Decimal

class wdgOpportunities(QWidget, Ui_wdgOpportunities):
    def __init__(self, mem,  parent=None):
        QWidget.__init__(self, parent)
        self.setupUi(self)
        self.mem=mem
        self.opportunities=None 
         
        self.txtInvest.setText(Decimal(self.mem.settingsdb.value("wdgIndexRange/invertir", "10000")))
        self.tblOpportunities.settings(self.mem, "wdgOpportunities")
        self.on_cmbMode_currentIndexChanged(self.cmbMode.currentIndex())
        self.wdgYear.initiate(self.opportunities.date_of_the_first_database_oppportunity().year,  datetime.date.today().year, datetime.date.today().year)
        
        
    @pyqtSlot()  
    def on_actionOpportunityNew_triggered(self):
        d=QDialog(self)     
        d.setModal(True)
        d.setWindowTitle(self.tr("Add new opportunity"))
        w=wdgOpportunitiesAdd(self.mem, None, d)
        lay = QVBoxLayout(d)
        lay.addWidget(w)
        d.exec_()    
        self.on_cmbMode_currentIndexChanged(self.cmbMode.currentIndex())
    
    @pyqtSlot()  
    def on_actionOpportunityEdit_triggered(self):
        d=QDialog(self)     
        d.setModal(True)
        d.setWindowTitle(self.tr("Edit opportunity"))
        w=wdgOpportunitiesAdd(self.mem, self.opportunities.selected, d)
        lay = QVBoxLayout(d)
        lay.addWidget(w)
        d.exec_()
        self.on_cmbMode_currentIndexChanged(self.cmbMode.currentIndex())
        
    @pyqtSlot() 
    def on_actionOpportunityDelete_triggered(self):
        self.opportunities.remove(self.opportunities.selected)
        self.mem.con.commit()
        self.on_cmbMode_currentIndexChanged(self.cmbMode.currentIndex())
        
        
    @pyqtSlot() 
    def on_actionExecute_triggered(self):        
        if self.opportunities.selected.executed==None:#Only adds an order if it's not executed
            d=QDialog(self)        
            d.setModal(True)
            d.setFixedSize(850, 850)
            d.setWindowTitle(self.tr("Investment calculator"))
            w=wdgCalculator(self.mem, self)
            w.setProduct(self.opportunities.selected.product)
            w.txtFinalPrice.setText(self.opportunities.selected.entry)
            lay = QVBoxLayout(d)
            lay.addWidget(w)
            d.exec_()
            reply = QMessageBox.question(None, self.tr('Opportunity execution'), self.tr("If you have make and order, you must execute\nDo you want to execute this oportunity?"),  QMessageBox.Yes, QMessageBox.No)          
            if reply==QMessageBox.Yes:
                self.opportunities.selected.executed=self.mem.localzone.now()#Set execution
                self.opportunities.selected.save()
                self.mem.con.commit()
        else:
            self.opportunities.selected.executed=None#Remove execution
            self.opportunities.selected.save()
            self.mem.con.commit()
        self.on_cmbMode_currentIndexChanged(self.cmbMode.currentIndex())

    @pyqtSlot() 
    def on_actionRemove_triggered(self):        
        self.opportunities.selected.removed=datetime.date.today()
        self.opportunities.selected.save()
        self.mem.con.commit()
        self.on_cmbMode_currentIndexChanged(self.cmbMode.currentIndex())

    @pyqtSlot() 
    def on_actionShowGraphic_triggered(self):
        self.opportunities.selected.product.needStatus(2)
        d=QDialog(self)     
        d.showMaximized()
        d.setWindowTitle(self.tr("Opportunity graph"))
        lay = QVBoxLayout(d)
        wc=wdgProductHistoricalOpportunity()
        wc.setProduct(self.opportunities.selected.product, None)
        wc.setOpportunity(self.opportunities.selected)
#        wc.setPrice(self.opportunities.selected.entry)
        wc.generate()
        wc.display()
        lay.addWidget(wc)
        d.exec_()

    def load_OpportunityData(self, frm):
        """Carga los datos de la orden en el frmInvestmentOperationsAdd"""
        if self.opportunities.selected.shares<0:
            frm.cmbTiposOperaciones.setCurrentIndex(frm.cmbTiposOperaciones.findData(5))#Sale
        else:
            frm.cmbTiposOperaciones.setCurrentIndex(frm.cmbTiposOperaciones.findData(4))#Purchase
        frm.txtAcciones.setText(self.opportunities.selected.shares)
        frm.wdg2CPrice.setTextA(self.opportunities.selected.entry)

    @pyqtSlot(int)     
    def on_cmbMode_currentIndexChanged(self, index):
        if index==0:#Current
            self.wdgYear.hide()            
            self.opportunities=OpportunityManager(self.mem).init__from_db("""
                SELECT * 
                FROM 
                    OPPORTUNITIES
                WHERE
                    REMOVED IS NULL AND
                    EXECUTED IS NULL
                ORDER BY DATE
           """)
            self.opportunities.order_by_percentage_from_current_price()
        elif index==1: #show expired
            self.wdgYear.show()
            self.opportunities=OpportunityManager(self.mem).init__from_db(self.mem.con.mogrify("""
                SELECT * 
                FROM 
                    OPPORTUNITIES
                WHERE
                    DATE BETWEEN '%s-1-1' AND '%s-12-31' AND
                    REMOVED<NOW()::DATE AND
                    EXECUTED IS NULL
                ORDER BY DATE
           """, (self.wdgYear.year, self.wdgYear.year)))
            self.opportunities.order_by_removed()
        elif index==2: #show executed
            self.wdgYear.show()
            self.opportunities=OpportunityManager(self.mem).init__from_db(self.mem.con.mogrify("""
                SELECT * 
                FROM 
                    OPPORTUNITIES
                WHERE
                    DATE BETWEEN '%s-1-1' AND '%s-12-31' AND
                    EXECUTED IS NOT NULL
                ORDER BY DATE
           """, (self.wdgYear.year, self.wdgYear.year)))
            self.opportunities.order_by_executed()
        else:
            self.wdgYear.show()
            self.opportunities=OpportunityManager(self.mem).init__from_db(self.mem.con.mogrify("""
                SELECT * 
                FROM 
                    OPPORTUNITIES
                WHERE
                    DATE BETWEEN '%s-1-1' AND '%s-12-31'
                ORDER BY DATE
           """, (self.wdgYear.year, self.wdgYear.year)))
            self.opportunities.order_by_date()
        self.opportunities.myqtablewidget(self.tblOpportunities, self.txtInvest.decimal())
       
    def on_tblOpportunities_customContextMenuRequested(self,  pos):
        if self.opportunities.selected==None:
            self.actionOpportunityDelete.setEnabled(False)
            self.actionOpportunityEdit.setEnabled(False)
            self.actionExecute.setEnabled(False)
            self.actionRemove.setEnabled(False)
            self.actionShowGraphic.setEnabled(False)
        else:
            self.actionOpportunityDelete.setEnabled(True)
            self.actionOpportunityEdit.setEnabled(True)
            self.actionExecute.setEnabled(True)
            self.actionRemove.setEnabled(True)
            if self.opportunities.selected.executed==None:
                self.actionExecute.setText("Execute opportunity")
            else:
                self.actionExecute.setText("Remove execution time")
            self.actionShowGraphic.setEnabled(True)
                
                
            
        menu=QMenu()
        menu.addAction(self.actionOpportunityNew)
        menu.addSeparator()
        menu.addAction(self.actionOpportunityEdit)
        menu.addAction(self.actionOpportunityDelete)
        menu.addSeparator()
        menu.addAction(self.actionExecute)      
        menu.addAction(self.actionRemove)        
        menu.addSeparator()
        menu.addAction(self.actionShowGraphic)
        menu.exec_(self.tblOpportunities.mapToGlobal(pos))

    def on_tblOpportunities_itemSelectionChanged(self):
        self.opportunities.selected=None
        for i in self.tblOpportunities.selectedItems():
            if i.column()==0:#only once per row
                self.opportunities.selected=self.opportunities.arr[i.row()]
                
    def on_wdgYear_changed(self):
        self.on_cmbMode_currentIndexChanged(self.cmbMode.currentIndex())
        



from xulpymoney.libxulpymoney import Assets, Money
import math
import datetime
from PyQt5.QtWidgets import QWidget
from xulpymoney.ui.Ui_wdgInvestmentClasses import Ui_wdgInvestmentClasses
from xulpymoney.ui.canvaschart import VCPie
from PyQt5.QtChart import QChart

class wdgInvestmentClasses(QWidget, Ui_wdgInvestmentClasses):
    def __init__(self, mem,  parent=None):
        QWidget.__init__(self, parent)
        self.setupUi(self)
        self.mem=mem
        self.balances={}#Variable que cachea todos los balances
        self.hoy=datetime.date.today()
        self.animations=True#Mostrar animaciones

        self.viewTPC=VCPie()
        self.layTPC.addWidget(self.viewTPC)     
        self.viewPCI=VCPie()
        self.layPCI.addWidget(self.viewPCI)      
        self.viewTipo=VCPie()
        self.layTipo.addWidget(self.viewTipo)      
        self.viewApalancado=VCPie()
        self.layApalancado.addWidget(self.viewApalancado)    
        self.viewCountry=VCPie()
        self.layCountry.addWidget(self.viewCountry)      
        self.viewProduct=VCPie()
        self.layProduct.addWidget(self.viewProduct)
        
        self.accounts=Assets(self.mem).saldo_todas_cuentas(self.hoy).local()
        self.tab.setCurrentIndex(2)

    def update(self, animations=None):
        """Update calcs and charts"""
        if self.animations==None:
            self.animations=QChart.AllAnimations
        self.animations=animations
        self.scriptTPC()
        self.scriptPCI()
        self.scriptTipos()
        self.scriptApalancado()
        self.scriptCountry()
        self.scriptProduct()

    def on_radCurrent_toggled(self, checked):
        self.update()

    def scriptTPC(self):
        self.viewTPC.clear(self.animations)
        self.viewTPC.setCurrency(self.mem.localcurrency)
        for r in range(0, 11):
            total=Money(self.mem, 0,  self.mem.localcurrency)
            for i in self.mem.data.investments_active().arr:
                if math.ceil(i.product.percentage/10.0)==r:
                    if self.radCurrent.isChecked():
                        total=total+i.balance().local()
                    else:
                        total=total+i.invertido().local()
            if r==0:
                total=total+self.accounts
            if total.isGTZero():
                self.viewTPC.appendData("{0}% variable".format(r*10), total.amount)
        if self.radCurrent.isChecked():
            self.viewTPC.chart.setTitle(self.tr("Investment current balance by variable percentage"))
        else:
            self.viewTPC.chart.setTitle(self.tr("Invested balance by variable percentage"))
        self.viewTPC.display()

    def scriptPCI(self):
        self.viewPCI.clear(self.animations)
        self.viewPCI.setCurrency(self.mem.localcurrency)
        for m in self.mem.investmentsmodes.arr:
            total=Money(self.mem, 0,  self.mem.localcurrency)
            for i in self.mem.data.investments_active().arr:
                if i.product.mode==m:
                    if self.radCurrent.isChecked():
                        total=total+i.balance().local()
                    else:
                        total=total+i.invertido().local()
            if m.id=='c':
                total=total+self.accounts
            self.viewPCI.appendData(m.name.upper(), total.amount)
        if self.radCurrent.isChecked():    
            self.viewPCI.chart.setTitle(self.tr("Investment current balance by Put / Call / Inline"))   
        else:
            self.viewPCI.chart.setTitle(self.tr("Invested balance by Put / Call / Inline"))   
        self.viewPCI.display()

    def scriptTipos(self):
        self.viewTipo.clear(self.animations)
        self.viewTipo.setCurrency(self.mem.localcurrency)
        
        for t in self.mem.types.arr:
            total=Money(self.mem, 0,  self.mem.localcurrency)
            for i in self.mem.data.investments_active().arr:
                if i.product.type==t:
                    if self.radCurrent.isChecked():
                        total=total+i.balance().local()
                    else:
                        total=total+i.invertido().local()
            if t.id==11:#Accounts
                total=total+self.accounts
            if total.isGTZero():
                if t.id==11:#Accounts
                    self.viewTipo.appendData(t.name.upper(), total.amount, True)
                else:
                    self.viewTipo.appendData(t.name.upper(), total.amount)
        if self.radCurrent.isChecked():    
            self.viewTipo.chart.setTitle(self.tr("Investment current balance by product type"))   
        else:
            self.viewTipo.chart.setTitle(self.tr("Invested balance by product type"))   
        self.viewTipo.display()

    def scriptApalancado(self):
        self.viewApalancado.clear(self.animations)
        self.viewApalancado.setCurrency(self.mem.localcurrency)
        
        for a in self.mem.leverages.arr:
            total=Money(self.mem, 0,  self.mem.localcurrency)
            for i in self.mem.data.investments_active().arr:
                if i.product.leveraged==a:
                    if self.radCurrent.isChecked():
                        total=total+i.balance().local()
                    else:
                        total=total+i.invertido().local()
            if a.id==0:#Accounts
                total=total+self.accounts
            if total.isGTZero():
                self.viewApalancado.appendData(a.name.upper(), total.amount)
        if self.radCurrent.isChecked():    
            self.viewApalancado.chart.setTitle(self.tr("Investment current balance by leverage"))
        else:
            self.viewApalancado.chart.setTitle(self.tr("Invested balance by leverage"))
        self.viewApalancado.display()
        
    def scriptCountry(self):
        self.viewCountry.clear(self.animations)
        self.viewCountry.setCurrency(self.mem.localcurrency)
        
        for c in self.mem.countries.arr:
            total=Money(self.mem, 0,  self.mem.localcurrency)
            for i in self.mem.data.investments_active().arr:
                if i.product.stockmarket.country==c:
                    if self.radCurrent.isChecked():
                        total=total+i.balance().local()
                    else:
                        total=total+i.invertido().local()
            if total.isGTZero():
                self.viewCountry.appendData(c.name.upper(), total.amount)
        if self.radCurrent.isChecked():    
            self.viewCountry.chart.setTitle(self.tr("Investment current balance by country"))   
        else:
            self.viewCountry.chart.setTitle(self.tr("Invested balance by country"))   
        self.viewCountry.display()

    def scriptProduct(self):
        self.viewProduct.clear(self.animations)
        self.viewProduct.setCurrency(self.mem.localcurrency)
        
        invs=self.mem.data.investments_active().setInvestments_merging_investments_with_same_product_merging_current_operations()
        invs.order_by_balance()
        for i in invs.arr:
            if self.radCurrent.isChecked():
                saldo=i.balance(type=3)
            else:
                saldo=i.invertido(type=3)
            self.viewProduct.appendData(i.name.replace("Virtual investment merging current operations of ", ""), saldo.amount)

        self.viewProduct.appendData(self.tr("Accounts"), self.accounts.amount, True)
        
        if self.radCurrent.isChecked():    
            self.viewProduct.chart.setTitle(self.tr("Investment current balance by product"))
        else:
            self.viewProduct.chart.setTitle(self.tr("Invested balance by product"))        
        self.viewProduct.display()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/wdgInvestmentClasses.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_wdgInvestmentClasses(object):
    def setupUi(self, wdgInvestmentClasses):
        wdgInvestmentClasses.setObjectName("wdgInvestmentClasses")
        wdgInvestmentClasses.resize(1026, 487)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(wdgInvestmentClasses.sizePolicy().hasHeightForWidth())
        wdgInvestmentClasses.setSizePolicy(sizePolicy)
        self._2 = QtWidgets.QHBoxLayout(wdgInvestmentClasses)
        self._2.setObjectName("_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.lbl = QtWidgets.QLabel(wdgInvestmentClasses)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.lbl.setFont(font)
        self.lbl.setAlignment(QtCore.Qt.AlignCenter)
        self.lbl.setObjectName("lbl")
        self.verticalLayout.addWidget(self.lbl)
        self.groupBox = QtWidgets.QGroupBox(wdgInvestmentClasses)
        self.groupBox.setObjectName("groupBox")
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout(self.groupBox)
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.radCurrent = QtWidgets.QRadioButton(self.groupBox)
        self.radCurrent.setChecked(True)
        self.radCurrent.setObjectName("radCurrent")
        self.horizontalLayout_5.addWidget(self.radCurrent)
        self.radInvested = QtWidgets.QRadioButton(self.groupBox)
        self.radInvested.setObjectName("radInvested")
        self.horizontalLayout_5.addWidget(self.radInvested)
        self.horizontalLayout_8.addLayout(self.horizontalLayout_5)
        self.verticalLayout.addWidget(self.groupBox)
        self.tab = QtWidgets.QTabWidget(wdgInvestmentClasses)
        self.tab.setObjectName("tab")
        self.tabTPCVariable = QtWidgets.QWidget()
        self.tabTPCVariable.setObjectName("tabTPCVariable")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.tabTPCVariable)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.layTPC = QtWidgets.QHBoxLayout()
        self.layTPC.setObjectName("layTPC")
        self.horizontalLayout_2.addLayout(self.layTPC)
        self.tab.addTab(self.tabTPCVariable, "")
        self.tabPCI = QtWidgets.QWidget()
        self.tabPCI.setObjectName("tabPCI")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.tabPCI)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.layPCI = QtWidgets.QVBoxLayout()
        self.layPCI.setObjectName("layPCI")
        self.horizontalLayout.addLayout(self.layPCI)
        self.tab.addTab(self.tabPCI, "")
        self.tabTipo = QtWidgets.QWidget()
        self.tabTipo.setObjectName("tabTipo")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.tabTipo)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.layTipo = QtWidgets.QVBoxLayout()
        self.layTipo.setObjectName("layTipo")
        self.horizontalLayout_3.addLayout(self.layTipo)
        self.tab.addTab(self.tabTipo, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.tab_2)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.layApalancado = QtWidgets.QVBoxLayout()
        self.layApalancado.setObjectName("layApalancado")
        self.horizontalLayout_4.addLayout(self.layApalancado)
        self.tab.addTab(self.tab_2, "")
        self.tab_3 = QtWidgets.QWidget()
        self.tab_3.setObjectName("tab_3")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout(self.tab_3)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.layCountry = QtWidgets.QHBoxLayout()
        self.layCountry.setObjectName("layCountry")
        self.horizontalLayout_6.addLayout(self.layCountry)
        self.tab.addTab(self.tab_3, "")
        self.tab_4 = QtWidgets.QWidget()
        self.tab_4.setObjectName("tab_4")
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout(self.tab_4)
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.layProduct = QtWidgets.QHBoxLayout()
        self.layProduct.setObjectName("layProduct")
        self.horizontalLayout_7.addLayout(self.layProduct)
        self.tab.addTab(self.tab_4, "")
        self.verticalLayout.addWidget(self.tab)
        self._2.addLayout(self.verticalLayout)

        self.retranslateUi(wdgInvestmentClasses)
        self.tab.setCurrentIndex(5)
        QtCore.QMetaObject.connectSlotsByName(wdgInvestmentClasses)

    def retranslateUi(self, wdgInvestmentClasses):
        _translate = QtCore.QCoreApplication.translate
        self.lbl.setText(_translate("wdgInvestmentClasses", "Investments report"))
        self.groupBox.setTitle(_translate("wdgInvestmentClasses", "Calculated charts with"))
        self.radCurrent.setText(_translate("wdgInvestmentClasses", "Current value"))
        self.radInvested.setText(_translate("wdgInvestmentClasses", "Invested balance"))
        self.tab.setTabText(self.tab.indexOf(self.tabTPCVariable), _translate("wdgInvestmentClasses", "By variable percentaje"))
        self.tab.setTabText(self.tab.indexOf(self.tabPCI), _translate("wdgInvestmentClasses", "By Put / Call / Inline"))
        self.tab.setTabText(self.tab.indexOf(self.tabTipo), _translate("wdgInvestmentClasses", "By investment class"))
        self.tab.setTabText(self.tab.indexOf(self.tab_2), _translate("wdgInvestmentClasses", "By leverage"))
        self.tab.setTabText(self.tab.indexOf(self.tab_3), _translate("wdgInvestmentClasses", "By country"))
        self.tab.setTabText(self.tab.indexOf(self.tab_4), _translate("wdgInvestmentClasses", "By investment product"))


import xulpymoney.images.xulpymoney_rc



from PyQt5.QtCore import QCoreApplication
QCoreApplication.translate('Core','Personal Management')
QCoreApplication.translate('Core','Cash')
QCoreApplication.translate('Core', 'Desconocido')
QCoreApplication.translate('Core', 'Inicio de Cuenta')
QCoreApplication.translate('Core', 'Supermercado')
QCoreApplication.translate('Core', 'Restaurante')
QCoreApplication.translate('Core', 'Transferencia. Origen')
QCoreApplication.translate('Core', 'Transferencia. Destino')
QCoreApplication.translate('Core', 'Impuestos. Devolución')
QCoreApplication.translate('Core', 'Gasolina')
QCoreApplication.translate('Core', 'Ocio')
QCoreApplication.translate('Core', 'Teléfono')
QCoreApplication.translate('Core', 'Nómina')
QCoreApplication.translate('Core', 'Televisión & Video & Cine')
QCoreApplication.translate('Core', 'Casa. Comunidad')
QCoreApplication.translate('Core', 'Universidad & Estudios')
QCoreApplication.translate('Core', 'Viajes (Billetes y autopistas)')
QCoreApplication.translate('Core', 'Hoteles')
QCoreApplication.translate('Core', 'Loterías')
QCoreApplication.translate('Core', 'Regalos')
QCoreApplication.translate('Core', 'Electricidad')
QCoreApplication.translate('Core', 'Gas')
QCoreApplication.translate('Core', 'Intereses Negativos')
QCoreApplication.translate('Core', 'Ropa')
QCoreApplication.translate('Core', 'Médicos ')
QCoreApplication.translate('Core', 'Intereses Positivos')
QCoreApplication.translate('Core', 'Loterías Premios')
QCoreApplication.translate('Core', 'Informática')
QCoreApplication.translate('Core', 'Coche')
QCoreApplication.translate('Core', 'Herencia')
QCoreApplication.translate('Core', 'Compra producto de inversión')
QCoreApplication.translate('Core', 'Pagas')
QCoreApplication.translate('Core', 'Invitaciones')
QCoreApplication.translate('Core', 'Otros Ingresos')
QCoreApplication.translate('Core', 'Casa. Compra')
QCoreApplication.translate('Core', 'Venta producto de inversión')
QCoreApplication.translate('Core', 'Casa. Infraestructura')
QCoreApplication.translate('Core', 'Impuestos. Pago')
QCoreApplication.translate('Core', 'Comisiones bancarias')
QCoreApplication.translate('Core', 'Dividendos')
QCoreApplication.translate('Core', 'Facturacion Tarjeta')
QCoreApplication.translate('Core', 'Añadido de acciones')
QCoreApplication.translate('Core', 'Ajustes negativos')
QCoreApplication.translate('Core', 'Casa. Compra pareja')
QCoreApplication.translate('Core', 'Préstamo (Me devuelven)')
QCoreApplication.translate('Core', 'Seguro')
QCoreApplication.translate('Core', 'Boda')
QCoreApplication.translate('Core', 'Regalos económicos recibidos')
QCoreApplication.translate('Core', 'Prima asistencia')
QCoreApplication.translate('Core', 'Otros Ingresos Pareja')
QCoreApplication.translate('Core', 'Casa. Asistenta')
QCoreApplication.translate('Core', 'Taxi')
QCoreApplication.translate('Core', 'Copas')
QCoreApplication.translate('Core', 'Gimnasio')
QCoreApplication.translate('Core', 'Divorcio')
QCoreApplication.translate('Core', 'Donaciones')
QCoreApplication.translate('Core', 'Garage')
QCoreApplication.translate('Core', 'Comisión de custodia de valores')
QCoreApplication.translate('Core', 'Casa. Alquiler')
QCoreApplication.translate('Core', 'Dividendos. Venta derechos')
QCoreApplication.translate('Core', 'Renta fija. Pago cupón corrido')
QCoreApplication.translate('Core', 'Pareja. Apoyo')
QCoreApplication.translate('Core', 'Renta fija. Cobro cupón corrido')
QCoreApplication.translate('Core', 'Renta fija. Pago cupón')
QCoreApplication.translate('Core', 'Credit card refund')
QCoreApplication.translate('Core', 'HL adjustment income')
QCoreApplication.translate('Core', 'HL adjustment expense')
QCoreApplication.translate('Core', 'HL Guarantee payment')
QCoreApplication.translate('Core', 'HL Guarantee return')
QCoreApplication.translate('Core', 'HL Operation commission')
QCoreApplication.translate('Core', 'HL Paid interest')
QCoreApplication.translate('Core', 'HL Received interest')
QCoreApplication.translate('Core', 'Rollover Paid')
QCoreApplication.translate('Core', 'Rollover Received')



from PyQt5.QtCore import Qt, pyqtSlot
from PyQt5.QtWidgets import QMenu, QTableWidgetItem, QWidget
from xulpymoney.ui.Ui_wdgAccounts import Ui_wdgAccounts
from xulpymoney.ui.frmTransfer import frmTransfer
from xulpymoney.ui.frmAccountsReport import frmAccountsReport
from xulpymoney.libxulpymoney import Money
from xulpymoney.libxulpymoneyfunctions import qmessagebox

class wdgAccounts(QWidget, Ui_wdgAccounts):
    def __init__(self, mem,  parent=None):
        QWidget.__init__(self, parent)
        self.setupUi(self)
        self.mem=mem
        self.tblAccounts.settings(self.mem, "wdgAccounts")
        self.accounts=self.mem.data.accounts_active()
        self.selAccount=None
        self.child=None#Used to access childs in automate unittests
        self.load_table()

    def load_table(self):
        """Función que carga la tabla de cuentas"""
        self.accounts.order_by_name()
        self.tblAccounts.applySettings()
        self.tblAccounts.setRowCount(self.accounts.length());
        sumsaldos=Money(self.mem, 0, self.mem.localcurrency)
        for i, c in enumerate(self.accounts.arr):
            self.tblAccounts.setItem(i, 0, QTableWidgetItem(c.name))
            self.tblAccounts.setItem(i, 1, QTableWidgetItem(c.eb.name))
            self.tblAccounts.setItem(i, 2, QTableWidgetItem(c.numero))
            balance=c.balance()
            self.tblAccounts.setItem(i, 3, balance.local().qtablewidgetitem())
            sumsaldos=sumsaldos+balance.local()
        self.lblTotal.setText(self.tr("Accounts balance: {0}".format(sumsaldos)))
        self.tblAccounts.clearSelection()
        
    @pyqtSlot() 
    def on_actionAccountReport_triggered(self):
        self.child=self.child=frmAccountsReport(self.mem,   self.selAccount, self)
        self.child.exec_()
        self.on_chkInactivas_stateChanged(self.chkInactivas.checkState())
        
    @pyqtSlot() 
    def on_actionAccountAdd_triggered(self):
        self.child=frmAccountsReport(self.mem, None)
        self.child.exec_()
        self.on_chkInactivas_stateChanged(self.chkInactivas.checkState())
      
    @pyqtSlot() 
    def on_actionAccountDelete_triggered(self):
        if self.selAccount.eb.qmessagebox_inactive() or self.selAccount.qmessagebox_inactive():
            return
        cur = self.mem.con.cursor()
        if self.selAccount.is_deletable()==False:
            qmessagebox(self.tr("This account has associated investments, credit cards or operations. It can't be deleted"))
        else:
            self.selAccount.borrar(cur)
            self.mem.con.commit()
            #Only can't be deleted an active account, so I remove from active set
            self.mem.data.accounts.remove(self.selAccount)
        cur.close()
        self.on_chkInactivas_stateChanged(self.chkInactivas.checkState())
        self.load_table()
        
    def on_chkInactivas_stateChanged(self, state):
        if state==Qt.Unchecked:
            self.accounts=self.mem.data.accounts_active()
        else:
             
            self.accounts=self.mem.data.accounts_inactive()
        self.load_table()
        

    def on_tblAccounts_customContextMenuRequested(self,  pos):
        if self.selAccount==None:
            self.actionAccountDelete.setEnabled(False)
            self.actionAccountReport.setEnabled(False)
            self.actionActive.setEnabled(False)
            self.actionAccountReport.setEnabled(False)
        else:
            self.actionAccountDelete.setEnabled(True)
            self.actionAccountReport.setEnabled(True)
            self.actionActive.setEnabled(True)
            self.actionAccountReport.setEnabled(True)
            self.actionActive.setChecked(self.selAccount.active)
        
        menu=QMenu()
        menu.addAction(self.actionAccountAdd)
        menu.addAction(self.actionAccountDelete)
        menu.addSeparator()
        menu.addAction(self.actionActive)
        menu.addSeparator()
        menu.addAction(self.actionTransfer)
        menu.addSeparator()
        menu.addAction(self.actionAccountReport)
        menu.exec_(self.tblAccounts.mapToGlobal(pos))

        
    @pyqtSlot() 
    def on_actionActive_triggered(self):
        if self.selAccount.eb.qmessagebox_inactive()==True:
            return
        
         #Debe tenerlas para borrarla luego
        self.selAccount.active=self.chkInactivas.isChecked()
        self.selAccount.save()
        self.mem.con.commit()     
        #Recoloca en los Setcuentas
#        if self.selAccount.active==True:#Está todavía en inactivas
#            self.mem.data.accounts_active().append(self.selAccount)
#            self.mem.data.accounts_inactive().remove(self.selAccount)
#        else:#Está todavía en activas
#            self.mem.data.accounts_active().remove(self.selAccount)
#            self.mem.data.accounts_inactive().append(self.selAccount)    
        self.on_chkInactivas_stateChanged(self.chkInactivas.checkState())
        self.load_table()

    @pyqtSlot()  
    def on_actionTransfer_triggered(self):
        self.child=frmTransfer(self.mem, self.selAccount)
        self.child.exec_()
        self.load_table()
        
    def on_tblAccounts_cellDoubleClicked(self, row,  column):
        self.on_actionAccountReport_triggered()
        
    def on_tblAccounts_itemSelectionChanged(self):
        self.selAccount=None
        for i in self.tblAccounts.selectedItems():#itera por cada item no row.
            if i.column()==0:
                self.selAccount=self.accounts.arr[i.row()]
                break



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/wdgConcepts.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_wdgConcepts(object):
    def setupUi(self, wdgConcepts):
        wdgConcepts.setObjectName("wdgConcepts")
        wdgConcepts.resize(754, 525)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/xulpymoney/books.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        wdgConcepts.setWindowIcon(icon)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(wdgConcepts)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.lblTotal = QtWidgets.QLabel(wdgConcepts)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.lblTotal.setFont(font)
        self.lblTotal.setText("")
        self.lblTotal.setAlignment(QtCore.Qt.AlignCenter)
        self.lblTotal.setObjectName("lblTotal")
        self.horizontalLayout_4.addWidget(self.lblTotal)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.lbl = QtWidgets.QLabel(wdgConcepts)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.lbl.setFont(font)
        self.lbl.setAlignment(QtCore.Qt.AlignCenter)
        self.lbl.setObjectName("lbl")
        self.verticalLayout.addWidget(self.lbl)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem)
        self.wdgYM = wdgYearMonth(wdgConcepts)
        self.wdgYM.setObjectName("wdgYM")
        self.horizontalLayout_3.addWidget(self.wdgYM)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.tab = QtWidgets.QTabWidget(wdgConcepts)
        self.tab.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tab.setTabsClosable(True)
        self.tab.setObjectName("tab")
        self.tabExpenses = QtWidgets.QWidget()
        self.tabExpenses.setObjectName("tabExpenses")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.tabExpenses)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.layExpenses = QtWidgets.QHBoxLayout()
        self.layExpenses.setObjectName("layExpenses")
        self.tblExpenses = myQTableWidget(self.tabExpenses)
        self.tblExpenses.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblExpenses.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tblExpenses.setAlternatingRowColors(True)
        self.tblExpenses.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tblExpenses.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tblExpenses.setObjectName("tblExpenses")
        self.tblExpenses.setColumnCount(4)
        self.tblExpenses.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tblExpenses.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblExpenses.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblExpenses.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblExpenses.setHorizontalHeaderItem(3, item)
        self.tblExpenses.verticalHeader().setVisible(False)
        self.tblExpenses.verticalHeader().setCascadingSectionResizes(False)
        self.layExpenses.addWidget(self.tblExpenses)
        self.verticalLayout_3.addLayout(self.layExpenses)
        self.tab.addTab(self.tabExpenses, "")
        self.tabIncomes = QtWidgets.QWidget()
        self.tabIncomes.setObjectName("tabIncomes")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.tabIncomes)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.layIncomes = QtWidgets.QHBoxLayout()
        self.layIncomes.setObjectName("layIncomes")
        self.tblIncomes = myQTableWidget(self.tabIncomes)
        self.tblIncomes.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblIncomes.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tblIncomes.setAlternatingRowColors(True)
        self.tblIncomes.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tblIncomes.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tblIncomes.setObjectName("tblIncomes")
        self.tblIncomes.setColumnCount(4)
        self.tblIncomes.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tblIncomes.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblIncomes.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblIncomes.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblIncomes.setHorizontalHeaderItem(3, item)
        self.tblIncomes.verticalHeader().setVisible(False)
        self.tblIncomes.verticalHeader().setCascadingSectionResizes(False)
        self.layIncomes.addWidget(self.tblIncomes)
        self.verticalLayout_2.addLayout(self.layIncomes)
        self.tab.addTab(self.tabIncomes, "")
        self.verticalLayout.addWidget(self.tab)
        self.horizontalLayout_4.addLayout(self.verticalLayout)
        self.actionHistoricalReport = QtWidgets.QAction(wdgConcepts)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/xulpymoney/history.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionHistoricalReport.setIcon(icon1)
        self.actionHistoricalReport.setObjectName("actionHistoricalReport")

        self.retranslateUi(wdgConcepts)
        self.tab.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(wdgConcepts)

    def retranslateUi(self, wdgConcepts):
        _translate = QtCore.QCoreApplication.translate
        self.lbl.setText(_translate("wdgConcepts", "Concepts report"))
        self.tblExpenses.setSortingEnabled(False)
        item = self.tblExpenses.horizontalHeaderItem(0)
        item.setText(_translate("wdgConcepts", "Concept"))
        item = self.tblExpenses.horizontalHeaderItem(1)
        item.setText(_translate("wdgConcepts", "Monthly expenses"))
        item = self.tblExpenses.horizontalHeaderItem(2)
        item.setText(_translate("wdgConcepts", "% Montly expenses"))
        item = self.tblExpenses.horizontalHeaderItem(3)
        item.setText(_translate("wdgConcepts", "Monthly average"))
        self.tab.setTabText(self.tab.indexOf(self.tabExpenses), _translate("wdgConcepts", "Expenses"))
        self.tblIncomes.setSortingEnabled(False)
        item = self.tblIncomes.horizontalHeaderItem(0)
        item.setText(_translate("wdgConcepts", "Concept"))
        item = self.tblIncomes.horizontalHeaderItem(1)
        item.setText(_translate("wdgConcepts", "Monthly expenses"))
        item = self.tblIncomes.horizontalHeaderItem(2)
        item.setText(_translate("wdgConcepts", "% Montly expenses"))
        item = self.tblIncomes.horizontalHeaderItem(3)
        item.setText(_translate("wdgConcepts", "Monthly average"))
        self.tab.setTabText(self.tab.indexOf(self.tabIncomes), _translate("wdgConcepts", "Incomes"))
        self.actionHistoricalReport.setText(_translate("wdgConcepts", "Historical report"))
        self.actionHistoricalReport.setToolTip(_translate("wdgConcepts", "Historical report"))


from xulpymoney.ui.myqtablewidget import myQTableWidget
from xulpymoney.ui.wdgYearMonth import wdgYearMonth
import xulpymoney.images.xulpymoney_rc



import sys
sys.path.append("/usr/lib/xulpymoney")
from xulpymoney.libxulpymoney import *
from xulpymoney.ui.frmMain import *
from xulpymoney.ui.frmInit import *
from xulpymoney.ui.wdgProductSelector import *

## Parameters simulation
class MemSimulation(MemXulpymoney):
    def __init__(self):
        MemXulpymoney.__init__(self)
        self.database="xulpymoney_simulation"
        self.database_real="xulpymoney"
        self.date_start=datetime.date(2005,1,1)
        self.ranges_percentaje=2
        self.money=Decimal(100000)
        self.index_lowest=4000
        self.strcon_real="dbname='{}' port='5432' user='postgres' host='127.0.0.1' password='*'".format(self.database_real)
        self.con_real=psycopg2.extras.DictConnection(self.strcon_real)
        
    def drop_db(self):
        strtemplate1="dbname='template1' port='5432' user='postgres' host='127.0.0.1' password='*'" 
        cont=psycopg2.extras.DictConnection(strtemplate1)
        cont.set_isolation_level(0)                                    
        try:
            cur=cont.cursor()
            cur.execute("drop database {};".format(self.database))
            cur.close()
            cont.close()
        except:
            cur.close()
            cont.close()
            return False
        return True        
        
        
    def investment_create(self, range):
        frmMain.on_actionInvestments_triggered()
        ##Add
        w=frmInvestmentReport(mem, None)
        w.cmbAccount.setCurrentIndex(w.cmbAccount.findData(5))
        w.txtInvestment.setText("Simulated Investment {}".format(int(range)))
        d=frmProductSelector(None, mem)
        d.txt.setText("79329")
        d.on_cmd_released()
        d.tblInvestments.setCurrentCell(0, 0)
        w.ise.setSelected(d.selected)
        w.on_cmdInvestment_pressed()
        
    def range_change(self, new):
        if new<current_range*1.02 or new>current_range*0.98:
           return False
        return True
        
    def range_already_invested(self, range):
        pass
        

init=datetime.datetime.now()
if __name__ == '__main__':
    #Create db pruebas
    app = QApplication(sys.argv)
    QTextCodec.setCodecForTr(QTextCodec.codecForName("UTF-8"));
    
    translator = QTranslator(app)
    locale=QLocale()
    a=locale.system().name()
    if len(a)!=2:
        a=a[:-len(a)+2]
    s= QApplication.translate("Core",  "Local language detected:{0}").format(a)
    print (s)
    translator.load("/usr/lib/xulpymoney/xulpymoney_" + a + ".qm")
    app.installTranslator(translator);

    mem=MemSimulation()
    frm = frmInit() 
    frm.txtXulpymoney.setText(mem.database)
    mem.drop_db() 
    if frm.create_db(frm.txtXulpymoney.text()) and frm.create_xulpymoney():
        #Load data Xulpymoney infraestructure
        mem.setQTranslator(QTranslator(app))
        mem.qtranslator.load("/usr/lib/xulpymoney/xulpymoney_{0}.qm".format(mem.language.id))
        app.installTranslator(mem.qtranslator)
        frmMain= frmMain(mem)
        strcon="dbname='{}' port='5432' user='postgres' host='127.0.0.1' password='*'".format(mem.database)
        mem.con=psycopg2.extras.DictConnection(strcon) 
        mem.load_db_data()
    else:
        print("No pude crear db")


    
    #Load quotes from real database
    print (mem.data.benchmark.id)
    quotes=[]
    cur=mem.con_real.cursor()
    cur.execute("select * from quotes where id=79329")
    for row in cur:
        quote=Quote(mem).init__db_row(row, mem.data.benchmark)
        quote.save()
        quotes.append(quote)
    cur.close()
    mem.con.commit()

    #Create bank
    frmMain.on_actionBanks_triggered()
    frmMain.w.bank_add("Simulated bank")#It will be bank 4
    
    #Create account
    frmMain.on_actionAccounts_triggered()
    w=frmAccountsReport(mem, None)
    w.cmbEB.setCurrentIndex(w.cmbEB.findData(4))
    w.txtAccount.setText("Simulated account")
    w.cmbCurrency.setCurrentIndex(w.cmbCurrency.findData("EUR"))
    w.txtNumero.setText("01234567890123456789")
    w.on_cmdDatos_released()#it will be account 5
    frmMain.w.on_chkInactivas_stateChanged(frmMain.w.chkInactivas.checkState())
    frmMain.w.load_table()

    current_range=0
    current_time=None
    mem.investment_create(current_range)
    #iterate quotes
    for quote in quotes:
        
        #Hay venta
        
        #Hay cambio de rango
        if mem.range_change(quote.quote):
            #Se ha invertido en rango
            if mem.range_already_invested(current_range):
                pass
        
    
        #Hay reinversion
    print ("Simulation duration: {}".format(datetime.datetime.now()-init))
    
    
    frmMain.show()
    
    sys.exit(app.exec_())    



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/frmHlContractAdd.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_frmHlContractAdd(object):
    def setupUi(self, frmHlContractAdd):
        frmHlContractAdd.setObjectName("frmHlContractAdd")
        frmHlContractAdd.setWindowModality(QtCore.Qt.WindowModal)
        frmHlContractAdd.resize(450, 290)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/xulpymoney/dividends.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        frmHlContractAdd.setWindowIcon(icon)
        frmHlContractAdd.setModal(True)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(frmHlContractAdd)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.lblTitulo = QtWidgets.QLabel(frmHlContractAdd)
        font = QtGui.QFont()
        font.setPointSize(13)
        font.setBold(True)
        font.setWeight(75)
        self.lblTitulo.setFont(font)
        self.lblTitulo.setStyleSheet("color: rgb(0, 128, 0);")
        self.lblTitulo.setAlignment(QtCore.Qt.AlignCenter)
        self.lblTitulo.setObjectName("lblTitulo")
        self.verticalLayout.addWidget(self.lblTitulo)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.wdgDT = wdgDatetime(frmHlContractAdd)
        self.wdgDT.setObjectName("wdgDT")
        self.horizontalLayout.addWidget(self.wdgDT)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.label = QtWidgets.QLabel(frmHlContractAdd)
        self.label.setObjectName("label")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label)
        self.txtGuarantee = myQLineEdit(frmHlContractAdd)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtGuarantee.sizePolicy().hasHeightForWidth())
        self.txtGuarantee.setSizePolicy(sizePolicy)
        self.txtGuarantee.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.txtGuarantee.setObjectName("txtGuarantee")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.txtGuarantee)
        self.lblLiquido_7 = QtWidgets.QLabel(frmHlContractAdd)
        self.lblLiquido_7.setObjectName("lblLiquido_7")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.lblLiquido_7)
        self.txtAdjustment = myQLineEdit(frmHlContractAdd)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtAdjustment.sizePolicy().hasHeightForWidth())
        self.txtAdjustment.setSizePolicy(sizePolicy)
        self.txtAdjustment.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.txtAdjustment.setObjectName("txtAdjustment")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.txtAdjustment)
        self.txtInterest = myQLineEdit(frmHlContractAdd)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtInterest.sizePolicy().hasHeightForWidth())
        self.txtInterest.setSizePolicy(sizePolicy)
        self.txtInterest.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.txtInterest.setObjectName("txtInterest")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.txtInterest)
        self.txtCommission = myQLineEdit(frmHlContractAdd)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtCommission.sizePolicy().hasHeightForWidth())
        self.txtCommission.setSizePolicy(sizePolicy)
        self.txtCommission.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.txtCommission.setObjectName("txtCommission")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.txtCommission)
        self.lblGross = QtWidgets.QLabel(frmHlContractAdd)
        self.lblGross.setObjectName("lblGross")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.lblGross)
        self.lblLiquido = QtWidgets.QLabel(frmHlContractAdd)
        self.lblLiquido.setObjectName("lblLiquido")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.lblLiquido)
        self.verticalLayout.addLayout(self.formLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem2)
        self.cmd = QtWidgets.QPushButton(frmHlContractAdd)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/xulpymoney/save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmd.setIcon(icon1)
        self.cmd.setObjectName("cmd")
        self.horizontalLayout_2.addWidget(self.cmd)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem3)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3.addLayout(self.verticalLayout)

        self.retranslateUi(frmHlContractAdd)
        QtCore.QMetaObject.connectSlotsByName(frmHlContractAdd)
        frmHlContractAdd.setTabOrder(self.txtAdjustment, self.txtGuarantee)
        frmHlContractAdd.setTabOrder(self.txtGuarantee, self.txtInterest)
        frmHlContractAdd.setTabOrder(self.txtInterest, self.txtCommission)
        frmHlContractAdd.setTabOrder(self.txtCommission, self.cmd)

    def retranslateUi(self, frmHlContractAdd):
        _translate = QtCore.QCoreApplication.translate
        frmHlContractAdd.setWindowTitle(_translate("frmHlContractAdd", "Daily high-low investment contract"))
        self.lblTitulo.setText(_translate("frmHlContractAdd", "Daily high-low investment contract"))
        self.label.setText(_translate("frmHlContractAdd", "Guarantee amount"))
        self.txtGuarantee.setText(_translate("frmHlContractAdd", "0"))
        self.lblLiquido_7.setText(_translate("frmHlContractAdd", "Adjustment amount"))
        self.txtAdjustment.setText(_translate("frmHlContractAdd", "0"))
        self.txtInterest.setText(_translate("frmHlContractAdd", "0"))
        self.txtCommission.setText(_translate("frmHlContractAdd", "0"))
        self.lblGross.setText(_translate("frmHlContractAdd", "Interest amount"))
        self.lblLiquido.setText(_translate("frmHlContractAdd", "Commission amount"))


from xulpymoney.ui.myqlineedit import myQLineEdit
from xulpymoney.ui.wdgDatetime import wdgDatetime
import xulpymoney.images.xulpymoney_rc



from PyQt5.QtCore import Qt, pyqtSlot
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QMenu, QTableWidgetItem, QWidget
from datetime import date
from xulpymoney.libxulpymoney import Money, Percentage
from xulpymoney.libxulpymoneyfunctions import qmessagebox, qright
from xulpymoney.ui.Ui_wdgDividendsReport import Ui_wdgDividendsReport
from xulpymoney.ui.frmInvestmentReport import frmInvestmentReport
from xulpymoney.ui.frmProductReport import frmProductReport
from xulpymoney.ui.frmEstimationsAdd import frmEstimationsAdd

class wdgDividendsReport(QWidget, Ui_wdgDividendsReport):
    def __init__(self, mem,  parent=None):
        QWidget.__init__(self, parent)
        self.setupUi(self)
        self.mem=mem
        self.investmentes=[]

        self.tblInvestments.settings(self.mem, "wdgDividendsReport")
        
        self.on_chkInactivas_stateChanged(Qt.Unchecked)

    @pyqtSlot()  
    def on_actionEstimationDPSEdit_triggered(self):
        d=frmEstimationsAdd(self.mem, self.selInvestment.product, "dps")
        d.exec_()
        self.on_chkInactivas_stateChanged(self.chkInactivas.checkState())
        self.tblInvestments.clearSelection()

    def on_chkInactivas_stateChanged(self,  state):               
        if state==Qt.Checked:
             
            self.investmentes=self.mem.data.investments_inactive()
        else:
            self.investmentes=self.mem.data.investments_active()
        self.load_inversiones()
        
    def load_inversiones(self):    
        if self.investmentes.order_by_dividend()==False:
            qmessagebox(self.tr("I couldn't order data due to they have null values"))     
        
        self.tblInvestments.applySettings()
        self.tblInvestments.clearContents()
        self.tblInvestments.setRowCount(len(self.investmentes.arr));
        sumdiv=Money(self.mem, 0, self.mem.localcurrency)
        for i, inv in enumerate(self.investmentes.arr):
            if inv.product.estimations_dps.find(date.today().year)==None:
                dpa=0
                tpc=Percentage()
                divestimado=Money(self.mem, 0, self.mem.localcurrency)
            else:
                dpa=inv.product.estimations_dps.currentYear().estimation
                tpc=inv.product.estimations_dps.currentYear().percentage()
                divestimado=inv.dividend_bruto_estimado().local()            
            self.tblInvestments.setItem(i, 0,QTableWidgetItem(inv.name))
            self.tblInvestments.setItem(i, 1, QTableWidgetItem(inv.account.eb.name))
            self.tblInvestments.setItem(i, 2, inv.product.currency.qtablewidgetitem(inv.product.result.basic.last.quote))
            self.tblInvestments.setItem(i, 3, inv.product.currency.qtablewidgetitem(dpa))    
            self.tblInvestments.setItem(i, 4, qright(inv.shares()))
            sumdiv=sumdiv+divestimado
            self.tblInvestments.setItem(i, 5, divestimado.qtablewidgetitem())
            self.tblInvestments.setItem(i, 6, tpc.qtablewidgetitem())
                
            #Colorea si está desactualizado
            if inv.product.estimations_dps.dias_sin_actualizar()>self.spin.value():
                self.tblInvestments.item(i, 3).setIcon(QIcon(":/xulpymoney/alarm_clock.png"))
        self.lblTotal.setText(self.tr("If I keep this investment during a year, I'll get {0}").format( sumdiv))
            
        
    @pyqtSlot() 
    def on_actionInvestmentReport_triggered(self):
        w=frmInvestmentReport(self.mem, self.selInvestment, self)
        w.exec_()
        self.on_chkInactivas_stateChanged(self.chkInactivas.checkState())

            
    @pyqtSlot() 
    def on_actionProductReport_triggered(self):
        w=frmProductReport(self.mem, self.selInvestment.product, self.selInvestment, self)
        w.exec_()
        self.on_chkInactivas_stateChanged(self.chkInactivas.checkState())

                    
    def on_cmd_pressed(self):
        self.on_chkInactivas_stateChanged(self.chkInactivas.checkState())

    def on_tblInvestments_customContextMenuRequested(self,  pos):        
        menu=QMenu()
        menu.addAction(self.actionEstimationDPSEdit)
        menu.addSeparator()
        menu.addAction(self.actionInvestmentReport) 
        menu.addAction(self.actionProductReport)    
        menu.exec_(self.tblInvestments.mapToGlobal(pos))

    def on_tblInvestments_itemSelectionChanged(self):
        self.selInvestment=None
        for i in self.tblInvestments.selectedItems():#itera por cada item no row.
            self.selInvestment=self.investmentes.arr[i.row()]
        print ("Seleccionado: " +  str(self.selInvestment))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/frmAuxiliarTables.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_frmAuxiliarTables(object):
    def setupUi(self, frmAuxiliarTables):
        frmAuxiliarTables.setObjectName("frmAuxiliarTables")
        frmAuxiliarTables.resize(612, 541)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/xulpymoney/configure.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        frmAuxiliarTables.setWindowIcon(icon)
        self.horizontalLayout = QtWidgets.QHBoxLayout(frmAuxiliarTables)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(frmAuxiliarTables)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.label.setAlignment(QtCore.Qt.AlignCenter)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.tabWidget = QtWidgets.QTabWidget(frmAuxiliarTables)
        self.tabWidget.setObjectName("tabWidget")
        self.tabConceptos = QtWidgets.QWidget()
        self.tabConceptos.setObjectName("tabConceptos")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.tabConceptos)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.tblConcepts = myQTableWidget(self.tabConceptos)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tblConcepts.sizePolicy().hasHeightForWidth())
        self.tblConcepts.setSizePolicy(sizePolicy)
        self.tblConcepts.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblConcepts.setAlternatingRowColors(True)
        self.tblConcepts.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tblConcepts.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tblConcepts.setObjectName("tblConcepts")
        self.tblConcepts.setColumnCount(2)
        self.tblConcepts.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tblConcepts.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblConcepts.setHorizontalHeaderItem(1, item)
        self.tblConcepts.verticalHeader().setVisible(False)
        self.verticalLayout_2.addWidget(self.tblConcepts)
        self.horizontalLayout_2.addLayout(self.verticalLayout_2)
        self.tabWidget.addTab(self.tabConceptos, "")
        self.verticalLayout.addWidget(self.tabWidget)
        self.horizontalLayout.addLayout(self.verticalLayout)
        self.actionExpensesAdd = QtWidgets.QAction(frmAuxiliarTables)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/xulpymoney/add.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionExpensesAdd.setIcon(icon1)
        self.actionExpensesAdd.setObjectName("actionExpensesAdd")
        self.actionConceptDelete = QtWidgets.QAction(frmAuxiliarTables)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/xulpymoney/eventdelete.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionConceptDelete.setIcon(icon2)
        self.actionConceptDelete.setObjectName("actionConceptDelete")
        self.actionIncomesAdd = QtWidgets.QAction(frmAuxiliarTables)
        self.actionIncomesAdd.setIcon(icon1)
        self.actionIncomesAdd.setObjectName("actionIncomesAdd")
        self.actionChangeName = QtWidgets.QAction(frmAuxiliarTables)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/xulpymoney/editar.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionChangeName.setIcon(icon3)
        self.actionChangeName.setObjectName("actionChangeName")

        self.retranslateUi(frmAuxiliarTables)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(frmAuxiliarTables)

    def retranslateUi(self, frmAuxiliarTables):
        _translate = QtCore.QCoreApplication.translate
        frmAuxiliarTables.setWindowTitle(_translate("frmAuxiliarTables", "Xulpymoney > Auxiliar tables"))
        self.label.setText(_translate("frmAuxiliarTables", "Auxiliar tables"))
        item = self.tblConcepts.horizontalHeaderItem(0)
        item.setText(_translate("frmAuxiliarTables", "Concepts"))
        item = self.tblConcepts.horizontalHeaderItem(1)
        item.setText(_translate("frmAuxiliarTables", "Operation type"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabConceptos), _translate("frmAuxiliarTables", "Xulpymoney concepts"))
        self.actionExpensesAdd.setText(_translate("frmAuxiliarTables", "Add a expense concept"))
        self.actionExpensesAdd.setToolTip(_translate("frmAuxiliarTables", "Add a expense concept"))
        self.actionConceptDelete.setText(_translate("frmAuxiliarTables", "Delete concept"))
        self.actionConceptDelete.setToolTip(_translate("frmAuxiliarTables", "Delete concept"))
        self.actionIncomesAdd.setText(_translate("frmAuxiliarTables", "Add an income concept"))
        self.actionIncomesAdd.setToolTip(_translate("frmAuxiliarTables", "Add an income concept"))
        self.actionChangeName.setText(_translate("frmAuxiliarTables", "Change name"))
        self.actionChangeName.setToolTip(_translate("frmAuxiliarTables", "Change name"))


from xulpymoney.ui.myqtablewidget import myQTableWidget
import xulpymoney.images.xulpymoney_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/frmInvestmentOperationsAdd.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_frmInvestmentOperationsAdd(object):
    def setupUi(self, frmInvestmentOperationsAdd):
        frmInvestmentOperationsAdd.setObjectName("frmInvestmentOperationsAdd")
        frmInvestmentOperationsAdd.setWindowModality(QtCore.Qt.WindowModal)
        frmInvestmentOperationsAdd.resize(719, 557)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/xulpymoney/document-edit.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        frmInvestmentOperationsAdd.setWindowIcon(icon)
        frmInvestmentOperationsAdd.setModal(True)
        self.horizontalLayout = QtWidgets.QHBoxLayout(frmInvestmentOperationsAdd)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.lblTitulo = QtWidgets.QLabel(frmInvestmentOperationsAdd)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.lblTitulo.setFont(font)
        self.lblTitulo.setStyleSheet("color: rgb(0, 128, 0);")
        self.lblTitulo.setAlignment(QtCore.Qt.AlignCenter)
        self.lblTitulo.setObjectName("lblTitulo")
        self.verticalLayout.addWidget(self.lblTitulo)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem)
        self.wdgDT = wdgDatetime(frmInvestmentOperationsAdd)
        self.wdgDT.setObjectName("wdgDT")
        self.horizontalLayout_8.addWidget(self.wdgDT)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout_8)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.lblType = QtWidgets.QLabel(frmInvestmentOperationsAdd)
        self.lblType.setObjectName("lblType")
        self.horizontalLayout_3.addWidget(self.lblType)
        self.cmbTiposOperaciones = QtWidgets.QComboBox(frmInvestmentOperationsAdd)
        self.cmbTiposOperaciones.setObjectName("cmbTiposOperaciones")
        self.horizontalLayout_3.addWidget(self.cmbTiposOperaciones)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.lblShares = QtWidgets.QLabel(frmInvestmentOperationsAdd)
        self.lblShares.setObjectName("lblShares")
        self.horizontalLayout_5.addWidget(self.lblShares)
        self.txtAcciones = myQLineEdit(frmInvestmentOperationsAdd)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtAcciones.sizePolicy().hasHeightForWidth())
        self.txtAcciones.setSizePolicy(sizePolicy)
        self.txtAcciones.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.txtAcciones.setObjectName("txtAcciones")
        self.horizontalLayout_5.addWidget(self.txtAcciones)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        self.wdg2CCurrencyConversion = wdgTwoCurrencyLineEdit(frmInvestmentOperationsAdd)
        self.wdg2CCurrencyConversion.setObjectName("wdg2CCurrencyConversion")
        self.verticalLayout.addWidget(self.wdg2CCurrencyConversion)
        self.wdg2CPrice = wdgTwoCurrencyLineEdit(frmInvestmentOperationsAdd)
        self.wdg2CPrice.setObjectName("wdg2CPrice")
        self.verticalLayout.addWidget(self.wdg2CPrice)
        self.wdg2CTaxes = wdgTwoCurrencyLineEdit(frmInvestmentOperationsAdd)
        self.wdg2CTaxes.setObjectName("wdg2CTaxes")
        self.verticalLayout.addWidget(self.wdg2CTaxes)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.wdg2CComission = wdgTwoCurrencyLineEdit(frmInvestmentOperationsAdd)
        self.wdg2CComission.setObjectName("wdg2CComission")
        self.horizontalLayout_2.addWidget(self.wdg2CComission)
        self.cmdComissionCalculator = QtWidgets.QToolButton(frmInvestmentOperationsAdd)
        self.cmdComissionCalculator.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/xulpymoney/tools-wizard.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmdComissionCalculator.setIcon(icon1)
        self.cmdComissionCalculator.setObjectName("cmdComissionCalculator")
        self.horizontalLayout_2.addWidget(self.cmdComissionCalculator)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.wdg2CGross = wdgTwoCurrencyLineEdit(frmInvestmentOperationsAdd)
        self.wdg2CGross.setObjectName("wdg2CGross")
        self.verticalLayout.addWidget(self.wdg2CGross)
        self.wdg2CNet = wdgTwoCurrencyLineEdit(frmInvestmentOperationsAdd)
        self.wdg2CNet.setObjectName("wdg2CNet")
        self.verticalLayout.addWidget(self.wdg2CNet)
        self.cmd = QtWidgets.QPushButton(frmInvestmentOperationsAdd)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/xulpymoney/save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmd.setIcon(icon2)
        self.cmd.setObjectName("cmd")
        self.verticalLayout.addWidget(self.cmd)
        self.horizontalLayout.addLayout(self.verticalLayout)

        self.retranslateUi(frmInvestmentOperationsAdd)
        QtCore.QMetaObject.connectSlotsByName(frmInvestmentOperationsAdd)

    def retranslateUi(self, frmInvestmentOperationsAdd):
        _translate = QtCore.QCoreApplication.translate
        frmInvestmentOperationsAdd.setWindowTitle(_translate("frmInvestmentOperationsAdd", "New investment operation"))
        self.lblType.setText(_translate("frmInvestmentOperationsAdd", "Operation type"))
        self.lblShares.setText(_translate("frmInvestmentOperationsAdd", "Number of shares"))
        self.txtAcciones.setText(_translate("frmInvestmentOperationsAdd", "0"))
        self.cmdComissionCalculator.setToolTip(_translate("frmInvestmentOperationsAdd", "Comission calculator"))
        self.cmd.setText(_translate("frmInvestmentOperationsAdd", "Save"))


from xulpymoney.ui.myqlineedit import myQLineEdit
from xulpymoney.ui.wdgDatetime import wdgDatetime
from xulpymoney.ui.wdgTwoCurrencyLineEdit import wdgTwoCurrencyLineEdit
import xulpymoney.images.xulpymoney_rc



from PyQt5.QtCore import Qt, pyqtSlot
from PyQt5.QtWidgets import QMenu, QWidget, QTableWidgetItem, QInputDialog, QLineEdit
from xulpymoney.libxulpymoney import Bank, Money, AccountManager, InvestmentManager
from xulpymoney.libxulpymoneyfunctions import wdgBool, qmessagebox
from xulpymoney.ui.frmAccountsReport import frmAccountsReport
from xulpymoney.ui.frmInvestmentReport import frmInvestmentReport
from xulpymoney.ui.Ui_wdgBanks import Ui_wdgBanks

class wdgBanks(QWidget, Ui_wdgBanks):
    def __init__(self, mem,  parent=None):
        QWidget.__init__(self, parent)
        self.setupUi(self)
        self.mem=mem
        
         
        self.banks=None#Set in on_chkActives_stateChanged
        self.investments=InvestmentManager(self.mem, self.mem.data.accounts, self.mem.data.products, self.mem.data.benchmark) #Set
        self.accounts=AccountManager(self.mem, self.mem.data.banks)#Set


        
        self.tblEB.settings(self.mem, "wdgBanks")
        self.tblAccounts.settings(self.mem, "wdgBanks")
        self.tblInvestments.settings(self.mem, "wdgBanks")
        
        self.on_chkActives_stateChanged(Qt.Checked)#Carga eb

        
    def load_eb(self):
        self.banks.order_by_name()
        self.tblEB.clearContents()
        self.tblEB.applySettings()
        self.tblEB.setRowCount(self.banks.length()+1)
        sumsaldos=Money(self.mem, 0, self.mem.localcurrency)
        for i,  e in enumerate(self.banks.arr):
            self.tblEB.setItem(i, 0, QTableWidgetItem(e.name))
            self.tblEB.setCellWidget(i, 1, wdgBool(e.active))
            balanc=e.balance(self.mem.data.accounts_active(), self.mem.data.investments_active())
            self.tblEB.setItem(i, 2, balanc.qtablewidgetitem())
            sumsaldos=sumsaldos+balanc    
        self.tblEB.setItem(self.banks.length(), 0, QTableWidgetItem(self.tr('TOTAL')))
        self.tblEB.setItem(self.banks.length(), 2, sumsaldos.qtablewidgetitem())
        
    def load_cuentas(self):
        self.accounts.order_by_name()
        self.tblAccounts.applySettings()
        self.tblAccounts.clearContents()
        self.tblAccounts.setRowCount(self.accounts.length()+1);
        sumsaldos=Money(self.mem, 0,  self.mem.localcurrency)
        for i,  c in enumerate(self.accounts.arr):
            self.tblAccounts.setItem(i, 0, QTableWidgetItem(c.name))
            self.tblAccounts.setCellWidget(i, 1, wdgBool(c.active))
            balance=c.balance().local()
            self.tblAccounts.setItem(i, 2, balance.qtablewidgetitem())
            sumsaldos=sumsaldos+balance
        self.tblAccounts.setItem(self.accounts.length(), 0, QTableWidgetItem(self.tr('TOTAL')))
        self.tblAccounts.setItem(self.accounts.length(), 2, sumsaldos.qtablewidgetitem())  
        
                
    def load_inversiones(self):
        self.investments.order_by_name()
        self.tblInvestments.applySettings()
        self.tblInvestments.clearContents()
        self.tblInvestments.setRowCount(self.investments.length()+1);
        sumsaldos=Money(self.mem,  0,  self.mem.localcurrency)
        for i, inv in enumerate(self.investments.arr):
            self.tblInvestments.setItem(i, 0, QTableWidgetItem(inv.name))
            self.tblInvestments.setCellWidget(i, 1, wdgBool(inv.active))
            balanc=inv.balance().local()
            self.tblInvestments.setItem(i, 2, balanc.qtablewidgetitem())
            sumsaldos=sumsaldos+balanc
        self.tblInvestments.setItem(self.investments.length(), 0, QTableWidgetItem(self.tr('TOTAL')))
        self.tblInvestments.setItem(self.investments.length(), 2, sumsaldos.qtablewidgetitem())
        
        
    def on_chkActives_stateChanged(self, state):
        self.banks=self.mem.data.banks_set(self.chkActives.isChecked())
        self.load_eb()
        self.tblEB.clearSelection()   
        self.tblAccounts.setRowCount(0)
        self.tblAccounts.clearContents()
        self.tblInvestments.setRowCount(0);
        self.tblInvestments.clearContents()

    def on_tblEB_itemSelectionChanged(self):
        self.banks.selected=None
        self.investments.clean()
        self.accounts.clean()
        
        for i in self.tblEB.selectedItems():#itera por cada item no row.
            if i.row()<self.banks.length():#Necesario porque tiene fila de total
                self.banks.selected=self.banks.arr[i.row()]
        print ("Seleccionado: " +  str(self.banks.selected))
        
        if self.banks.selected==None:
            self.tblEB.clearSelection()   
            self.tblAccounts.setRowCount(0)
            self.tblAccounts.clearContents()
            self.tblInvestments.setRowCount(0);
            self.tblInvestments.clearContents()
            return
                   
        for i in self.mem.data.investments.arr:
            if i.account.eb.id==self.banks.selected.id:
                if (self.chkActives.isChecked() and i.active==True) or (self.chkActives.isChecked()==False):
                    self.investments.append(i)
        
        for v in self.mem.data.accounts.arr:
            if v.eb.id==self.banks.selected.id:
                if (self.chkActives.isChecked() and v.active==True) or (self.chkActives.isChecked()==False):
                    self.accounts.append(v)
        
        self.load_cuentas()
        self.load_inversiones()

    def on_tblEB_customContextMenuRequested(self,  pos):
        if self.banks.selected==None:
            self.actionBankDelete.setEnabled(False)
            self.actionBankEdit.setEnabled(False)
            self.actionActive.setEnabled(False)
        else:
            self.actionBankDelete.setEnabled(True)
            self.actionBankEdit.setEnabled(True)
            self.actionActive.setEnabled(True)
            self.actionActive.setChecked(self.banks.selected.active)

        menu=QMenu()
        menu.addAction(self.actionBankAdd)
        menu.addAction(self.actionBankEdit)
        menu.addAction(self.actionBankDelete)
        menu.addSeparator()
        menu.addAction(self.actionActive)
        menu.exec_(self.tblEB.mapToGlobal(pos))
        
    @pyqtSlot() 
    def on_actionAccountReport_triggered(self):
        w=frmAccountsReport(self.mem, self.accounts.selected)
        w.exec_()        
        self.load_cuentas()

    @pyqtSlot() 
    def on_actionInvestmentReport_triggered(self):
        w=frmInvestmentReport(self.mem,   self.investments.selected)
        w.exec_()
        self.load_inversiones()

    def on_tblAccounts_itemSelectionChanged(self):
        self.accounts.selected=None
        for i in self.tblAccounts.selectedItems():#itera por cada item no row.
            if i.row()<self.accounts.length():#Necesario porque tiene fila de total
                self.accounts.selected=self.accounts.arr[i.row()]
        print ("Seleccionado: " +  str(self.accounts.selected))

    def on_tblAccounts_customContextMenuRequested(self,  pos):
        if self.accounts.selected==None:
            self.actionAccountReport.setEnabled(False)
        else:
            self.actionAccountReport.setEnabled(True)
        menu=QMenu()
        menu.addAction(self.actionAccountReport)
        menu.exec_(self.tblAccounts.mapToGlobal(pos))


    def on_tblInvestments_itemSelectionChanged(self):
        self.investments.selected=None
        for i in self.tblInvestments.selectedItems():#itera por cada item no row.
            if i.row()<self.investments.length():#Necesario porque tiene fila de total
                self.investments.selected=self.investments.arr[i.row()]
        print ("Seleccionado: " +  str(self.investments.selected))

        
    def on_tblInvestments_customContextMenuRequested(self,  pos):
        if self.investments.selected==None:
            self.actionInvestmentReport.setEnabled(False)
        else:
            self.actionInvestmentReport.setEnabled(True)
        menu=QMenu()
        menu.addAction(self.actionInvestmentReport)        
        menu.exec_(self.tblInvestments.mapToGlobal(pos))
        
    @pyqtSlot()  
    def on_actionBankDelete_triggered(self):
        if self.banks.selected.qmessagebox_inactive():
            return        
        if self.banks.selected.is_deletable()==False:
            qmessagebox(self.tr("This bank has dependent accounts and it can't be deleted"))
        else:
            self.mem.data.banks.delete(self.banks.selected)
            self.mem.con.commit()  
            self.on_chkActives_stateChanged(self.chkActives.checkState())    
        
    @pyqtSlot()  
    def on_actionBankAdd_triggered(self):
        tipo=QInputDialog().getText(self,  "Xulpymoney",  self.tr("Add a new bank"))
        if tipo[1]==True:
            self.bank_add(tipo[0])
            
    def bank_add(self, bank):
        """Permits unit tests if separated"""
        eb=Bank(self.mem).init__create(bank)
        eb.save()
        self.mem.con.commit()  
        self.mem.data.banks.append(eb)
        self.on_chkActives_stateChanged(self.chkActives.checkState())


    @pyqtSlot()  
    def on_actionBankEdit_triggered(self):
        tipo=QInputDialog().getText(self,  "Xulpymoney", self.tr("Edit selected bank") , QLineEdit.Normal,   (self.banks.selected.name))       
        if tipo[1]==True:
            self.bank_edit(tipo[0])
            
    def bank_edit(self, bank):
        """Permits unit tests if separated"""
        self.banks.selected.name=bank
        self.banks.selected.save()
        self.mem.con.commit()
        self.on_chkActives_stateChanged(self.chkActives.checkState())   
        
    @pyqtSlot() 
    def on_actionActive_triggered(self):
        self.banks.selected.active=not self.banks.selected.active
        self.banks.selected.save()
        self.mem.con.commit()   
        
        #Recoloca en los InvestmentManager
        print (self.banks.selected)
#        if self.banks.selected.active==True:#Está todavía en inactivas
#            self.mem.data.banks_active().append(self.banks.selected)
#            self.mem.data.banks_inactive().remove(self.banks.selected)
#        else:#Está todavía en activas
#            self.mem.data.banks_active().remove(self.banks.selected)
#            self.mem.data.banks_inactive().append(self.banks.selected)
        self.on_chkActives_stateChanged(self.chkActives.checkState())
